@@ -1,4 +1,4 @@
-/*  $Id: innxmit.c,v 1.68 2003/02/11 09:40:19 alexk Exp $
+/*  $Id: innxmit.c,v 1.68.2.2 2004/05/16 20:26:53 rra Exp $
 **
 **  Transmit articles to remote site.
 **  Modified for NNTP streaming: 1996-01-03 Jerry Aguirre
@@ -590,15 +590,16 @@ REMsendarticle(char *Article, char *MessageID, ARTHANDLE *art) {
 	vec[0].iov_base = (char *) art->data;
 	vec[0].iov_len = len;
 	/* Add 14 bytes, which maybe will be the length of the Bytes header */
-	snprintf(buf, sizeof(buf), "Bytes: %d\r\n", art->len + 14);
+	snprintf(buf, sizeof(buf), "Bytes: %lu\r\n",
+                 (unsigned long) art->len + 14);
 	vec[1].iov_base = buf;
 	vec[1].iov_len = strlen(buf);
 	if (iscmsg) {
 	    vec[2].iov_base = (char *) art->data + len;
 	    vec[2].iov_len = art->len - len;
 	} else {
-	    vec[2].iov_base = (char *) ".\r\n";
-	    vec[2].iov_len = 3;
+	    vec[2].iov_base = (char *) "\r\n.\r\n";
+	    vec[2].iov_len = 5;
 	}
 	if (xwritev(ToServer, vec, 3) < 0)
 	    return false;
@@ -608,7 +609,7 @@ REMsendarticle(char *Article, char *MessageID, ARTHANDLE *art) {
     if (GotInterrupt)
 	Interrupted(Article, MessageID);
     if (Debug) {
-	fprintf(stderr, "> [ article %d ]\n", art->len);
+	fprintf(stderr, "> [ article %lu ]\n", (unsigned long) art->len);
 	fprintf(stderr, "> .\n");
     }
 
