@@ -1,14 +1,15 @@
-/*  $Id: nntpget.c,v 1.14.2.1 2000/06/21 08:30:09 rra Exp $
+/*  $Id: nntpget.c,v 1.33 2003/01/19 01:15:26 rra Exp $
 **
 **  Connect to a remote site, and get news from it to offer to our local
 **  server.  Read list on stdin, or get it via NEWNEWS command.  Writes
 **  list of articles still needed to stdout.
 */
+
 #include "config.h"
 #include "clibrary.h"
+#include "portable/socket.h"
+#include "portable/time.h"
 #include <errno.h>
-#include <syslog.h>  
-#include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/uio.h>
 
@@ -17,20 +18,10 @@
 # include <sys/select.h>
 #endif
 
-#ifdef TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# ifdef HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
-
-#include "dbz.h"
+#include "inn/history.h"
+#include "inn/innconf.h"
+#include "inn/messages.h"
 #include "libinn.h"
-#include "macros.h"
 #include "nntp.h"
 #include "paths.h"
 
@@ -50,28 +41,27 @@ typedef struct _SITE {
 /*
 **  Global variables.
 */
-STATIC struct iovec	SITEvec[2];
-STATIC char		SITEv1[] = "\r\n";
-STATIC char		READER[] = "mode reader";
-STATIC unsigned long	STATgot;
-STATIC unsigned long	STAToffered;
-STATIC unsigned long	STATsent;
-STATIC unsigned long	STATrejected;
+static struct iovec	SITEvec[2];
+static char		SITEv1[] = "\r\n";
+static char		READER[] = "mode reader";
+static unsigned long	STATgot;
+static unsigned long	STAToffered;
+static unsigned long	STATsent;
+static unsigned long	STATrejected;
+static struct history	*History;
 
 
 
 /*
 **  Read a line of input, with timeout.
 */
-STATIC BOOL
-SITEread(sp, start)
-    SITE		*sp;
-    char		*start;
+static bool
+SITEread(SITE *sp, char *start)
 {
-    register char	*p;
-    register char	*end;
+    char	*p;
+    char	*end;
     struct timeval	t;
-    FDSET		rmask;
+    fd_set		rmask;
     int			i;
     char		c;
 
@@ -83,18 +73,18 @@ SITEread(sp, start)
 	    FD_SET(sp->Rfd, &rmask);
 	    t.tv_sec = DEFAULT_TIMEOUT;
 	    t.tv_usec = 0;
-	    i = select(sp->Rfd + 1, &rmask, (FDSET *)NULL, (FDSET *)NULL, &t);
+	    i = select(sp->Rfd + 1, &rmask, NULL, NULL, &t);
 	    if (i < 0) {
 		if (errno == EINTR)
 		    goto Again;
-		return FALSE;
+		return false;
 	    }
 	    if (i == 0
 	     || !FD_ISSET(sp->Rfd, &rmask)
 	     || (sp->Count = read(sp->Rfd, sp->Buffer, sizeof sp->Buffer)) < 0)
-		return FALSE;
+		return false;
 	    if (sp->Count == 0)
-		return FALSE;
+		return false;
 	    sp->bp = sp->Buffer;
 	}
 
@@ -111,7 +101,7 @@ SITEread(sp, start)
     if (p > start && p < end && p[-1] == '\r')
 	p--;
     *p = '\0';
-    return TRUE;
+    return true;
 }
 
 
@@ -120,21 +110,17 @@ SITEread(sp, start)
 **  since it's only for sending DATA to local site, and the data we got from
 **  the remote site already is escaped.
 */
-STATIC BOOL
-SITEwrite(sp, p, i)
-    SITE		*sp;
-    char		*p;
-    int			i;
+static bool
+SITEwrite(SITE *sp, const char *p, int i)
 {
-    SITEvec[0].iov_base = p;
+    SITEvec[0].iov_base = (char *) p;
     SITEvec[0].iov_len = i;
     return xwritev(sp->Wfd, SITEvec, 2) >= 0;
 }
 
 
-STATIC SITE *
-SITEconnect(host)
-    char	*host;
+static SITE *
+SITEconnect(char *host)
 {
     FILE	*From;
     FILE	*To;
@@ -146,28 +132,18 @@ SITEconnect(host)
 	i = NNTPconnect(host, NNTP_PORT, &From, &To, (char *)NULL);
     else {
 	host = innconf->server;
-        if (host == NULL) {
-            fprintf(stderr,
-                    "No server specified and server not set in inn.conf\n");
-            exit(1);
-        }
+        if (host == NULL)
+            die("no server specified and server not set in inn.conf");
 	i = NNTPlocalopen(&From, &To, (char *)NULL);
     }
-    if (i < 0) {
-	(void)fprintf(stderr, "Can't connect to \"%s\", %s\n",
-		host, strerror(errno));
-	exit(1);
-    }
+    if (i < 0)
+        sysdie("cannot connect to %s", host);
 
-    if (NNTPsendpassword(host, From, To) < 0) {
-	(void)fprintf(stderr, "Can't authenticate with %s, %s\n",
-		host, strerror(errno));
-	/* Don't send quit; we want the remote to print a message. */
-	exit(1);
-    }
+    if (NNTPsendpassword(host, From, To) < 0)
+        sysdie("cannot authenticate to %s", host);
 
     /* Build the structure. */
-    sp = NEW(SITE, 1);
+    sp = xmalloc(sizeof(SITE));
     sp->Name = host;
     sp->Rfd = fileno(From);
     sp->Wfd = fileno(To);
@@ -180,79 +156,76 @@ SITEconnect(host)
 /*
 **  Send "quit" to a site, and get its reply.
 */
-STATIC void
-SITEquit(sp)
-    SITE	*sp;
+static void
+SITEquit(SITE *sp)
 {
     char	buff[NNTP_STRLEN];
 
-    (void)SITEwrite(sp, "quit", 4);
-    (void)SITEread(sp, buff);
+    SITEwrite(sp, "quit", 4);
+    SITEread(sp, buff);
 }
 
 
-STATIC BOOL
-HIShaveit(mesgid)
-    char		*mesgid;
+static bool
+HIShaveit(char *mesgid)
 {
-    return dbzexists(HashMessageID(mesgid));
+    return HIScheck(History, mesgid);
 }
 
 
-STATIC NORETURN
-Usage(p)
-    char	*p;
+static void
+Usage(const char *p)
 {
-    (void)fprintf(stderr, "Usage error:  %s\n", p);
-    (void)fprintf(stderr,
-    "Usage:  nntpget [ -d dist -n grps [-f file | -t time -u file]] host\n");
+    warn("%s", p);
+    fprintf(stderr, "Usage: nntpget"
+            " [ -d dist -n grps [-f file | -t time -u file]] host\n");
     exit(1);
 }
 
 
 int
-main(ac, av)
-    int		ac;
-    char	*av[];
+main(int ac, char *av[])
 {
     char	buff[NNTP_STRLEN];
     char	mesgid[NNTP_STRLEN];
     char	tbuff[SMBUF];
-    char	temp[BUFSIZ];
-    STRING	Groups;
+    char	*msgidfile = NULL;
+    int         msgidfd;
+    const char	*Groups;
     char	*distributions;
     char	*Since;
+    char        *path;
     int		i;
     struct tm	*gt;
     struct stat	Sb;
     SITE	*Remote;
-    SITE	*Local;
+    SITE	*Local = NULL;
     FILE	*F;
-    BOOL	Offer;
-    BOOL	Error;
-    BOOL	Verbose = FALSE;
+    bool	Offer;
+    bool	Error;
+    bool	Verbose = false;
     char	*Update;
     char	*p;
 
-    /* First thing, set up logging and our identity. */
-    openlog("nntpget", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);           
+    /* First thing, set up our identity. */
+    message_program_name = "nntpget";
 
     /* Set defaults. */
     distributions = NULL;
     Groups = NULL;
     Since = NULL;
-    Offer = FALSE;
+    Offer = false;
     Update = NULL;
+    if (!innconf_read(NULL))
+        exit(1);
 
-    if (ReadInnConf() < 0) exit(1);
-
-    (void)umask(NEWSUMASK);
+    umask(NEWSUMASK);
 
     /* Parse JCL. */
     while ((i = getopt(ac, av, "d:f:n:t:ovu:")) != EOF)
 	switch (i) {
 	default:
-	    Usage("Bad flag");
+	    Usage("bad flag");
 	    /* NOTREACHED */
 	case 'd':
 	    distributions = optarg;
@@ -262,15 +235,12 @@ main(ac, av)
 	    /* FALLTHROUGH */
 	case 'f':
 	    if (Since)
-		Usage("Only one -f -t or -u flag");
-	    if (stat(optarg, &Sb) < 0) {
-		(void)fprintf(stderr, "Can't stat \"%s\", %s\n",
-			optarg, strerror(errno));
-		exit(1);
-	    }
+		Usage("only one of -f, -t, or -u may be given");
+	    if (stat(optarg, &Sb) < 0)
+                sysdie("cannot stat %s", optarg);
 	    gt = gmtime(&Sb.st_mtime);
 	    /* Y2K: NNTP Spec currently allows only two digit years. */
-	    (void)sprintf(tbuff, "%02d%02d%02d %02d%02d%02d GMT",
+	    snprintf(tbuff, sizeof(tbuff), "%02d%02d%02d %02d%02d%02d GMT",
 		    gt->tm_year % 100, gt->tm_mon + 1, gt->tm_mday,
 		    gt->tm_hour, gt->tm_min, gt->tm_sec);
 	    Since = tbuff;
@@ -280,116 +250,102 @@ main(ac, av)
 	    break;
 	case 'o':
 	    /* Open the history file. */
-	    if (!dbzinit(cpcatpath(innconf->pathdb, _PATH_HISTORY))) {
-		(void)fprintf(stderr, "Can't open history, %s\n",
-		    strerror(errno));
-		exit(1);
-	    }
-	    Offer = TRUE;
+            path = concatpath(innconf->pathdb, _PATH_HISTORY);
+	    History = HISopen(path, innconf->hismethod, HIS_RDONLY);
+	    if (!History)
+                sysdie("cannot open history");
+            free(path);
+	    Offer = true;
 	    break;
 	case 't':
 	    if (Since)
-		Usage("Only one -t or -f flag");
+		Usage("only one of -t or -f may be given");
 	    Since = optarg;
 	    break;
 	case 'v':
-	    Verbose = TRUE;
+	    Verbose = true;
 	    break;
 	}
     ac -= optind;
     av += optind;
     if (ac != 1)
-	Usage("No host given");
+	Usage("no host given");
 
     /* Set up the scatter/gather vectors used by SITEwrite. */
     SITEvec[1].iov_base = SITEv1;
-    SITEvec[1].iov_len = STRLEN(SITEv1);
+    SITEvec[1].iov_len = strlen(SITEv1);
 
     /* Connect to the remote server. */
-    if ((Remote = SITEconnect(av[0])) == NULL) {
-	(void)fprintf(stderr, "Can't connect to \"%s\", %s\n",
-		av[0], strerror(errno));
-	exit(1);
-    }
-    if (!SITEwrite(Remote, READER, (int)STRLEN(READER))
-     || !SITEread(Remote, buff)) {
-	(void)fprintf(stderr, "Can't start reading, %s\n", strerror(errno));
-	exit(1);
-    }
+    if ((Remote = SITEconnect(av[0])) == NULL)
+        sysdie("cannot connect to %s", av[0]);
+    if (!SITEwrite(Remote, READER, (int)strlen(READER))
+     || !SITEread(Remote, buff))
+        sysdie("cannot start reading");
 
     if (Since == NULL) {
 	F = stdin;
-	temp[0] = '\0';
 	if (distributions || Groups)
-	    Usage("No -d or -n when reading stdin");
+	    Usage("no -d or -n flags allowed when reading stdin");
     }
     else {
 	/* Ask the server for a list of what's new. */
 	if (Groups == NULL)
 	    Groups = "*";
 	if (distributions)
-	    (void)sprintf(buff, "NEWNEWS %s %s <%s>",
-		    Groups, Since, distributions);
+	    snprintf(buff, sizeof(buff), "NEWNEWS %s %s <%s>",
+                     Groups, Since, distributions);
 	else
-	    (void)sprintf(buff, "NEWNEWS %s %s", Groups, Since);
+	    snprintf(buff, sizeof(buff), "NEWNEWS %s %s", Groups, Since);
 	if (!SITEwrite(Remote, buff, (int)strlen(buff))
-	 || !SITEread(Remote, buff)) {
-	    (void)fprintf(stderr, "Can't start list, %s\n", strerror(errno));
-	    exit(1);
-	}
+	 || !SITEread(Remote, buff))
+            sysdie("cannot start list");
 	if (buff[0] != NNTP_CLASS_OK) {
-	    (void)fprintf(stderr, "Protocol error from \"%s\", got \"%s\"\n",
-		    Remote->Name, buff);
 	    SITEquit(Remote);
-	    exit(1);
+            die("protocol error from %s, got %s", Remote->Name, buff);
 	}
 
-	/* Create a temporary file. */
-	(void)sprintf(temp, "%s/nntpgetXXXXXX", innconf->pathtmp);
-	(void)mktemp(temp);
-	if ((F = fopen(temp, "w+")) == NULL) {
-	    (void)fprintf(stderr, "Can't open \"%s\", %s\n",
-		    temp, strerror(errno));
-	    exit(1);
-	}
+        /* Create a temporary file. */
+        msgidfile = concatpath(innconf->pathtmp, "nntpgetXXXXXX");
+        msgidfd = mkstemp(msgidfile);
+        if (msgidfd < 0)
+            sysdie("cannot create a temporary file");
+        F = fopen(msgidfile, "w+");
+        if (F == NULL)
+            sysdie("cannot open %s", msgidfile);
 
 	/* Read and store the Message-ID list. */
 	for ( ; ; ) {
 	    if (!SITEread(Remote, buff)) {
-		(void)fprintf(stderr, "Can't read from \"%s\", %s\n",
-			Remote->Name, strerror(errno));
-		(void)fclose(F);
+                syswarn("cannot read from %s", Remote->Name);
+		fclose(F);
 		SITEquit(Remote);
 		exit(1);
 	    }
-	    if (EQ(buff, "."))
+	    if (strcmp(buff, ".") == 0)
 		break;
 	    if (Offer && HIShaveit(buff))
 		continue;
 	    if (fprintf(F, "%s\n", buff) == EOF || ferror(F)) {
-		(void)fprintf(stderr, "Can't write \"%s\", %s\n",
-			temp, strerror(errno));
-		(void)fclose(F);
+                syswarn("cannot write %s", msgidfile);
+		fclose(F);
 		SITEquit(Remote);
 		exit(1);
 	    }
 	}
 	if (fflush(F) == EOF) {
-	    (void)fprintf(stderr, "Can't flush \"%s\", %s\n",
-		    temp, strerror(errno));
-	    (void)fclose(F);
+            syswarn("cannot flush %s", msgidfile);
+	    fclose(F);
 	    SITEquit(Remote);
 	    exit(1);
 	}
-	(void)fseek(F, (OFFSET_T)0, SEEK_SET);
+	fseeko(F, 0, SEEK_SET);
     }
 
     if (Offer) {
 	/* Connect to the local server. */
 	if ((Local = SITEconnect((char *)NULL)) == NULL) {
-	    (void)fprintf(stderr, "Can't connect to local server, %s\n", 
-		    strerror(errno));
-	    (void)fclose(F);
+            syswarn("cannot connect to local server");
+	    fclose(F);
 	    exit(1);
 	}
     }
@@ -403,11 +359,10 @@ main(ac, av)
 	if (Offer) {
 	    /* See if the local server wants it. */
 	    STAToffered++;
-	    (void)sprintf(buff, "ihave %s", mesgid);
+	    snprintf(buff, sizeof(buff), "ihave %s", mesgid);
 	    if (!SITEwrite(Local, buff, (int)strlen(buff))
 	     || !SITEread(Local, buff)) {
-		(void)fprintf(stderr, "Can't offer \"%s\", %s\n.",
-			mesgid, strerror(errno));
+                syswarn("cannot offer %s", mesgid);
 		break;
 	    }
 	    if (atoi(buff) != NNTP_SENDIT_VAL)
@@ -415,20 +370,18 @@ main(ac, av)
 	}
 
 	/* Try to get the article. */
-	(void)sprintf(buff, "article %s", mesgid);
+	snprintf(buff, sizeof(buff), "article %s", mesgid);
 	if (!SITEwrite(Remote, buff, (int)strlen(buff))
 	 || !SITEread(Remote, buff)) {
-	    (void)fprintf(stderr, "Can't get \"%s\", %s\n",
-		    mesgid, strerror(errno));
-	    (void)printf("%s\n", mesgid);
+            syswarn("cannot get %s", mesgid);
+	    printf("%s\n", mesgid);
 	    break;
 	}
 	if (atoi(buff) != NNTP_ARTICLE_FOLLOWS_VAL) {
           if (Offer) {
-              (void)SITEwrite(Local, ".", 1);
+              SITEwrite(Local, ".", 1);
               if (!SITEread(Local, buff)) {
-                  (void)fprintf(stderr, "No reply after \"%s\", %s\n",
-                                mesgid, strerror(errno));
+                  syswarn("no reply after %s", mesgid);
                   break;
               }
           }
@@ -436,31 +389,29 @@ main(ac, av)
 	}
 
 	if (Verbose)
-	    (void)fprintf(stderr, "%s...\n", mesgid);
+            notice("%s...", mesgid);
 
 	/* Read each line in the article and write it. */
-	for (Error = FALSE; ; ) {
+	for (Error = false; ; ) {
 	    if (!SITEread(Remote, buff)) {
-		(void)fprintf(stderr, "Can't read \"%s\" from \"%s\", %s\n",
-			mesgid, Remote->Name, strerror(errno));
-		Error = TRUE;
+                syswarn("cannot read %s from %s", mesgid, Remote->Name);
+		Error = true;
 		break;
 	    }
 	    if (Offer) {
 		if (!SITEwrite(Local, buff, (int)strlen(buff))) {
-		    (void)fprintf(stderr, "Can't send \"%s\", %s\n",
-			    mesgid, strerror(errno));
-		    Error = TRUE;
+                    syswarn("cannot send %s", mesgid);
+		    Error = true;
 		    break;
 		}
 	    }
 	    else
-		(void)printf("%s\n", buff);
-	    if (EQ(buff, "."))
+		printf("%s\n", buff);
+	    if (strcmp(buff, ".") == 0)
 		break;
 	}
 	if (Error) {
-	    (void)printf("%s\n", mesgid);
+	    printf("%s\n", mesgid);
 	    break;
 	}
 	STATsent++;
@@ -468,19 +419,18 @@ main(ac, av)
 	/* How did the local server respond? */
 	if (Offer) {
 	    if (!SITEread(Local, buff)) {
-		(void)fprintf(stderr, "No reply after \"%s\", %s\n",
-			mesgid, strerror(errno));
-		(void)printf("%s\n", mesgid);
+                syswarn("no reply after %s", mesgid);
+		printf("%s\n", mesgid);
 		break;
 	    }
 	    i = atoi(buff);
 	    if (i == NNTP_TOOKIT_VAL)
 		continue;
 	    if (i == NNTP_RESENDIT_VAL) {
-		(void)printf("%s\n", mesgid);
+		printf("%s\n", mesgid);
 		break;
 	    }
-	    (void)fprintf(stderr, "%s to \"%s\"\n", buff, mesgid);
+            syswarn("%s to %s", buff, mesgid);
 	    STATrejected++;
 	}
     }
@@ -490,15 +440,14 @@ main(ac, av)
 	while (fgets(mesgid, sizeof mesgid, F) != NULL) {
 	    if ((p = strchr(mesgid, '\n')) != NULL)
 		*p = '\0';
-	    (void)printf("%s\n", mesgid);
+	    printf("%s\n", mesgid);
 	    STATgot++;
 	}
-    (void)fclose(F);
+    fclose(F);
 
     /* Remove our temp file. */
-    if (temp[0] && unlink(temp) < 0)
-	(void)fprintf(stderr, "Can't remove \"%s\", %s\n",
-		temp, strerror(errno));
+    if (msgidfile && unlink(msgidfile) < 0)
+        syswarn("cannot remove %s", msgidfile);
 
     /* All done. */
     SITEquit(Remote);
@@ -507,18 +456,12 @@ main(ac, av)
 
     /* Update timestamp file? */
     if (Update) {
-	if ((F = fopen(Update, "w")) == NULL) {
-	    (void)fprintf(stderr, "Can't update %s, %s\n",
-		    Update, strerror(errno));
-	    exit(1);
-	}
-	(void)fprintf(F, "got %ld offered %ld sent %ld rejected %ld\n",
+	if ((F = fopen(Update, "w")) == NULL)
+            sysdie("cannot update %s", Update);
+	fprintf(F, "got %ld offered %ld sent %ld rejected %ld\n",
 		STATgot, STAToffered, STATsent, STATrejected); 
-	if (ferror(F) || fclose(F) == EOF) {
-	    (void)fprintf(stderr, "Can't update %s, %s\n",
-		    Update, strerror(errno));
-	    exit(1);
-	}
+	if (ferror(F) || fclose(F) == EOF)
+            sysdie("cannot update %s", Update);
     }
 
     exit(0);
