@@ -1,4 +1,4 @@
-/*  $Id: rnews.c,v 1.24.2.2 2001/01/08 07:11:05 rra Exp $
+/*  $Id: rnews.c,v 1.54.2.4 2004/06/03 23:52:56 rra Exp $
 **
 **  A front-end for InterNetNews.
 **
@@ -9,37 +9,39 @@
 
 #include "config.h"
 #include "clibrary.h"
+#include "portable/wait.h"
 #include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
+#include <fcntl.h>
+#include <pwd.h>
 #include <syslog.h>
 #include <sys/stat.h>
 
-#ifdef HAVE_FCNTL_H
-# include <fcntl.h>
-#endif
-
+#include "inn/innconf.h"
+#include "inn/messages.h"
+#include "inn/wire.h"
 #include "libinn.h"
-#include "macros.h"
 #include "nntp.h"
 #include "paths.h"
+#include "storage.h"
 
 
 typedef struct _HEADER {
-    STRING	Name;
-    int		size;
+    const char *Name;
+    int size;
 } HEADER;
 
 
-STATIC BOOL	Verbose;
-STATIC char	*InputFile = "stdin";
-STATIC char	*UUCPHost;
-STATIC char	*PathBadNews = NULL;
-STATIC char	*remoteServer;
-STATIC FILE	*FromServer;
-STATIC FILE	*ToServer;
-STATIC char	UNPACK[] = "gzip";
-STATIC HEADER	RequiredHeaders[] = {
+static bool	Verbose;
+static const char	*InputFile = "stdin";
+static char	*UUCPHost;
+static char	*PathBadNews = NULL;
+static char	*remoteServer;
+static FILE	*FromServer;
+static FILE	*ToServer;
+static char	UNPACK[] = "gzip";
+static HEADER	RequiredHeaders[] = {
     { "Message-ID",	10 },
 #define _messageid	0
     { "Newsgroups",	10 },
@@ -58,81 +60,60 @@ STATIC HEADER	RequiredHeaders[] = {
 
 
 
-/*
-**  Do perror, making sure errno is preserved.
-*/
-STATIC void
-xperror(p)
-    char	*p;
-{
-    int		oerrno;
-
-    oerrno = errno;
-    perror(p);
-    errno = oerrno;
-}
-
-
 /*
 **  Open up a pipe to a process with fd tied to its stdin.  Return a
 **  descriptor tied to its stdout or -1 on error.
 */
-STATIC int
-StartChild(fd, path, argv)
-    int		fd;
-    char	*path;
-    char	*argv[];
+static int
+StartChild(int fd, const char *path, const char *argv[])
 {
     int		pan[2];
     int		i;
-    PID_T	pid;
+    pid_t	pid;
 
     /* Create a pipe. */
-    if (pipe(pan) < 0) {
-	syslog(L_FATAL, "cant pipe for %s %m", path);
-	exit(1);
-    }
+    if (pipe(pan) < 0)
+        sysdie("cannot pipe for %s", path);
 
     /* Get a child. */
-    for (i = 0; (pid = FORK()) < 0; i++) {
+    for (i = 0; (pid = fork()) < 0; i++) {
 	if (i == innconf->maxforks) {
-	    syslog(L_ERROR, "cant fork %s %m -- spooling", path);
+            syswarn("cannot fork %s, spooling", path);
 	    return -1;
 	}
-	syslog(L_NOTICE, "cant fork %s -- waiting", path);
-	(void)sleep(60);
+        notice("cannot fork %s, waiting", path);
+	sleep(60);
     }
 
     /* Run the child, with redirection. */
     if (pid == 0) {
-	(void)close(pan[PIPE_READ]);
+	close(pan[PIPE_READ]);
 
 	/* Stdin comes from our old input. */
-	if (fd != STDIN) {
-	    if ((i = dup2(fd, STDIN)) != STDIN) {
-		syslog(L_FATAL, "cant dup2 %d to 0 got %d %m", fd, i);
+	if (fd != STDIN_FILENO) {
+	    if ((i = dup2(fd, STDIN_FILENO)) != STDIN_FILENO) {
+                syswarn("cannot dup2 %d to 0, got %d", fd, i);
 		_exit(1);
 	    }
-	    (void)close(fd);
+	    close(fd);
 	}
 
 	/* Stdout goes down the pipe. */
-	if (pan[PIPE_WRITE] != STDOUT) {
-	    if ((i = dup2(pan[PIPE_WRITE], STDOUT)) != STDOUT) {
-		syslog(L_FATAL, "cant dup2 %d to 1 got %d %m",
-		    pan[PIPE_WRITE], i);
+	if (pan[PIPE_WRITE] != STDOUT_FILENO) {
+	    if ((i = dup2(pan[PIPE_WRITE], STDOUT_FILENO)) != STDOUT_FILENO) {
+                syswarn("cannot dup2 %d to 1, got %d", pan[PIPE_WRITE], i);
 		_exit(1);
 	    }
-	    (void)close(pan[PIPE_WRITE]);
+	    close(pan[PIPE_WRITE]);
 	}
 
-	(void)execv(path, argv);
-	syslog(L_FATAL, "cant execv %s %m", path);
+	execv(path, (char * const *)argv);
+        syswarn("cannot execv %s", path);
 	_exit(1);
     }
 
-    (void)close(pan[PIPE_WRITE]);
-    (void)close(fd);
+    close(pan[PIPE_WRITE]);
+    close(fd);
     return pan[PIPE_READ];
 }
 
@@ -140,20 +121,18 @@ StartChild(fd, path, argv)
 /*
 **  Wait for the specified number of children.
 */
-STATIC void
-WaitForChildren(i)
-    register int	i;
+static void
+WaitForChildren(int n)
 {
-    register PID_T	pid;
-    int			status;
+    pid_t pid;
 
-    while (--i >= 0) {
-	pid = waitnb(&status);
-	if (pid < 0) {
-	    if (errno != ECHILD)
-		syslog(L_ERROR, "cant wait %m");
-	    break;
-	}
+    while (--n >= 0) {
+        pid = waitpid(-1, NULL, WNOHANG);
+        if (pid == (pid_t) -1 && errno != EINTR) {
+            if (errno != ECHILD)
+                syswarn("cannot wait");
+            break;
+        }
     }
 }
 
@@ -163,7 +142,7 @@ WaitForChildren(i)
 /*
 **  Clean up the NNTP escapes from a line.
 */
-STATIC char *REMclean(char *buff)
+static char *REMclean(char *buff)
 {
     char	*p;
 
@@ -180,46 +159,57 @@ STATIC char *REMclean(char *buff)
 /*
 **  Write an article to the rejected directory.
 */
-STATIC void Reject(const char *article, const char *reason, const char *arg)
+static void Reject(const char *article, const char *reason, const char *arg)
 {
 #if	defined(DO_RNEWS_SAVE_BAD)
-    char	buff[SMBUF];
-    FILE	*F;
-    int		i;
+    char *filename;
+    FILE *F;
+    int fd;
+    size_t length;
 #endif	/* defined(DO_RNEWS_SAVE_BAD) */
 
-    syslog(L_NOTICE, reason, arg);
+    notice(reason, arg);
     if (Verbose) {
-	(void)fprintf(stderr, "%s: ", InputFile);
-	(void)fprintf(stderr, reason, arg);
-	(void)fprintf(stderr, " [%.40s...]\n", article);
+	fprintf(stderr, "%s: ", InputFile);
+	fprintf(stderr, reason, arg);
+	fprintf(stderr, " [%.40s...]\n", article);
     }
 
 #if	defined(DO_RNEWS_SAVE_BAD)
-    TempName(PathBadNews, buff);
-    if ((F = fopen(buff, "w")) == NULL) {
-	syslog(L_ERROR, "cant fopen %s %m", buff);
+    filename = concat(PathBadNews, "/XXXXXX", (char *) 0);
+    fd = mkstemp(filename);
+    if (fd < 0) {
+        warn("cannot create temporary file");
+        return;
+    }
+    F = fdopen(fd, "w");
+    if (F == NULL) {
+        warn("cannot fdopen %s", filename);
 	return;
     }
-    i = strlen(article);
-    if (fwrite((POINTER)article, (SIZE_T)1, (SIZE_T)i, F) != i)
-	syslog(L_ERROR, "cant fwrite %s %m", buff);
+    length = strlen(article);
+    if (fwrite(article, 1, length, F) != length)
+        warn("cannot fwrite to %s", filename);
     if (fclose(F) == EOF)
-	syslog(L_ERROR, "cant close %s %m", buff);
+        warn("cannot close %s", filename);
+    free(filename);
 #endif	/* defined(DO_RNEWS_SAVE_BAD) */
 }
 
 
 /*
-**  Process one article.  Return TRUE if the article was okay; FALSE if the
+**  Process one article.  Return true if the article was okay; false if the
 **  whole batch needs to be saved (such as when the server goes down or if
 **  the file is corrupted).
 */
-STATIC BOOL Process(char *article)
+static bool Process(char *article)
 {
     HEADER	        *hp;
-    char	        *p;
-    char		*id;
+    const char	        *p;
+    size_t              length;
+    char                *wirefmt, *q;
+    const char		*id = NULL;
+    char                *msgid;
     char		buff[SMBUF];
 #if	defined(FILE_RNEWS_LOG_DUPS)
     FILE		*F;
@@ -230,13 +220,18 @@ STATIC BOOL Process(char *article)
 
     /* Empty article? */
     if (*article == '\0')
-	return TRUE;
+	return true;
+
+    /* Convert the article to wire format. */
+    wirefmt = ToWireFmt(article, strlen(article), &length);
 
     /* Make sure that all the headers are there, note the ID. */
-    for (hp = RequiredHeaders; hp < ENDOF(RequiredHeaders); hp++) {
-	if ((p = (char *)HeaderFindMem(article, strlen(article), hp->Name, hp->size)) == NULL) {
+    for (hp = RequiredHeaders; hp < ARRAY_END(RequiredHeaders); hp++) {
+        p = wire_findheader(wirefmt, length, hp->Name);
+        if (p == NULL) {
+            free(wirefmt);
 	    Reject(article, "bad_article missing %s", hp->Name);
-	    return FALSE;
+	    return false;
 	}
 	if (IS_MESGID(hp)) {
 	    id = p;
@@ -244,154 +239,167 @@ STATIC BOOL Process(char *article)
 	}
 #if	!defined(DONT_RNEWS_LOG_DUPS)
 	if (IS_PATH(hp)) {
-	    (void)strncpy(path, p, sizeof path);
-	    path[sizeof path - 1] = '\0';
-	    if ((p = strchr(path, '\n')) != NULL)
-		*p = '\0';
+	    strlcpy(path, p, sizeof(path));
+	    if ((q = strchr(path, '\r')) != NULL)
+		*q = '\0';
 	}
 #endif	/* !defined(DONT_RNEWS_LOG_DUPS) */
     }
 
     /* Send the NNTP "ihave" message. */
-    if ((p = strchr(id, '\n')) == NULL) {
+    if ((p = strchr(id, '\r')) == NULL) {
+        free(wirefmt);
 	Reject(article, "bad_article unterminated %s header", "Message-ID");
-	return FALSE;
+	return false;
     }
-    *p = '\0';
-    (void)fprintf(ToServer, "ihave %s\r\n", id);
-    (void)fflush(ToServer);
+    msgid = xstrndup(id, p - id);
+    fprintf(ToServer, "ihave %s\r\n", msgid);
+    fflush(ToServer);
     if (UUCPHost)
-	syslog(L_NOTICE, "offered %s %s", id, UUCPHost);
-    *p = '\n';
+        notice("offered %s %s", msgid, UUCPHost);
+    free(msgid);
 
     /* Get a reply, see if they want the article. */
     if (fgets(buff, sizeof buff, FromServer) == NULL) {
-	syslog(L_ERROR, "cant fgets after ihave %m");
-	return FALSE;
+        free(wirefmt);
+        if (ferror(FromServer))
+            syswarn("cannot fgets after ihave");
+        else
+            warn("unexpected EOF from server after ihave");
+	return false;
     }
-    (void)REMclean(buff);
+    REMclean(buff);
     if (!CTYPE(isdigit, buff[0])) {
-	syslog(L_NOTICE, "bad_reply after ihave %s", buff);
-	return FALSE;
+        free(wirefmt);
+        notice("bad_reply after ihave %s", buff);
+	return false;
     }
     switch (atoi(buff)) {
     default:
+        free(wirefmt);
 	Reject(article, "unknown_reply after ihave %s", buff);
-	return TRUE;
+	return true;
     case NNTP_RESENDIT_VAL:
-	return FALSE;
+        free(wirefmt);
+	return false;
     case NNTP_SENDIT_VAL:
 	break;
     case NNTP_HAVEIT_VAL:
 #if	defined(SYSLOG_RNEWS_LOG_DUPS)
 	*p = '\0';
-	syslog(L_NOTICE, "duplicate %s %s", id, path);
+        notice("duplicate %s %s", id, path);
 #endif	/* defined(SYSLOG_RNEWS_LOG_DUPS) */
 #if	defined(FILE_RNEWS_LOG_DUPS)
 	if ((F = fopen(_PATH_RNEWS_DUP_LOG, "a")) != NULL) {
 	    *p = '\0';
-	    (void)fprintf(F, "duplicate %s %s\n", id, path);
-	    (void)fclose(F);
+	    fprintf(F, "duplicate %s %s\n", id, path);
+	    fclose(F);
 	}
 #endif	/* defined(FILE_RNEWS_LOG_DUPS) */
-	return TRUE;
+        free(wirefmt);
+	return true;
     }
 
-    /* Send all the lines in the article, escaping periods. */
-    if (NNTPsendarticle(article, ToServer, TRUE) < 0) {
-	syslog(L_NOTICE, "cant sendarticle %m");
-	return FALSE;
+    /* Send the article to the server. */
+    if (fwrite(wirefmt, length, 1, ToServer) != 1) {
+        free(wirefmt);
+        sysnotice("cant sendarticle");
+	return false;
+    }
+    free(wirefmt);
+
+    /* Flush the server buffer. */
+    if (fflush(ToServer) == EOF) {
+        syswarn("cant fflush after article");
+        return false;
     }
 
     /* Process server reply code. */
     if (fgets(buff, sizeof buff, FromServer) == NULL) {
-	syslog(L_ERROR, "cant fgets after article %m");
-	return FALSE;
+        if (ferror(FromServer))
+            syswarn("cannot fgets after article");
+        else
+            warn("unexpected EOF from server after article");
+	return false;
     }
-    (void)REMclean(buff);
+    REMclean(buff);
     if (!CTYPE(isdigit, buff[0])) {
-	syslog(L_NOTICE, "bad_reply after article %s", buff);
-	return FALSE;
+        notice("bad_reply after article %s", buff);
+	return false;
     }
     switch (atoi(buff)) {
     default:
-	syslog(L_NOTICE, "unknown_reply after article %s", buff);
+        notice("unknown_reply after article %s", buff);
 	/* FALLTHROUGH */
     case NNTP_RESENDIT_VAL:
-	return FALSE;
+	return false;
     case NNTP_TOOKIT_VAL:
 	break;
     case NNTP_REJECTIT_VAL:
 	Reject(article, "rejected %s", buff);
 	break;
     }
-    return TRUE;
+    return true;
 }
 
 
 /*
-**  Read the rest of the input as an article.  Just punt to stdio in
-**  this case and let it do the buffering.
+**  Read the rest of the input as an article.
 */
-STATIC BOOL
-ReadRemainder(fd, first, second)
-    register int	fd;
-    char		first;
-    char		second;
+static bool
+ReadRemainder(int fd, char first, char second)
 {
-    register FILE	*F;
-    register char	*article;
-    register char       *p;
-    register int	size;
-    register int	used;
-    register int	left;
-    register int	i;
-    BOOL		ok;
-
-    /* Turn the descriptor into a stream. */
-    if ((F = fdopen(fd, "r")) == NULL) {
-	syslog(L_FATAL, "can't fdopen %d %m", fd);
-	exit(1);
-    }
+    char	*article;
+    char	*p;
+    char	buf[BUFSIZ];
+    int		size;
+    int		used;
+    int		left;
+    int		skipnl;
+    int		i, n;
+    bool	ok;
 
     /* Get an initial allocation, leaving space for the \0. */
     size = BUFSIZ + 1;
-    article = NEW(char, size + 2);
+    article = xmalloc(size + 2);
     article[0] = first;
     article[1] = second;
     used = second ? 2 : 1;
     left = size - used;
+    skipnl = 0;
 
-    /* Read the input.  Read a line at a time so that we can convert line
-       endings if necessary. */
-    p = fgets(article + used, left, F);
-    while (p != NULL) {
-	i = strlen(p);
-        if (p[i-2] == '\r') {
-            p[i-2] = '\n';
-            p[i-1] = '\0';
-        }
-	used += i;
-        left -= i;
-        if (left < SMBUF) {
-            size += BUFSIZ;
-            left += BUFSIZ;
-            RENEW(article, char, size);
-        }
-        p = fgets(article + used, left, F);
-    }
-    if (!feof(F)) {
-        syslog(L_FATAL, "cant fgets after %d bytes %m", used);
-        exit(1);
+    /* Read the input, coverting line ends as we go if necessary. */
+    while ((n = read(fd, buf, sizeof(buf))) > 0) {
+	p = article + used;
+	for (i = 0; i < n; i++) {
+	    if (skipnl) {
+		skipnl = 0;
+		if (buf[i] == '\n') continue;
+	    }
+	    if (buf[i] == '\r') {
+		buf[i] = '\n';
+		skipnl = 1;
+	    }
+	    *p++ = buf[i];
+	    used++;
+	    left--;
+	    if (left < SMBUF) {
+		size += BUFSIZ;
+		left += BUFSIZ;
+		article = xrealloc(article, size);
+		p = article + used;
+	    }
+	}
     }
+    if (n < 0)
+        sysdie("cannot read after %d bytes", used);
 
     if (article[used - 1] != '\n')
 	article[used++] = '\n';
     article[used] = '\0';
-    (void)fclose(F);
 
     ok = Process(article);
-    DISPOSE(article);
+    free(article);
     return ok;
 }
 
@@ -399,34 +407,31 @@ ReadRemainder(fd, first, second)
 /*
 **  Read an article from the input stream that is artsize bytes long.
 */
-STATIC BOOL
-ReadBytecount(fd, artsize)
-    register int	fd;
-    int			artsize;
+static bool
+ReadBytecount(int fd, int artsize)
 {
     static char		*article;
     static int		oldsize;
-    register char	*p;
-    register int	left;
-    register int	i;
+    char	*p;
+    int	left;
+    int	i;
 
     /* If we haven't gotten any memory before, or we didn't get enough,
      * then get some. */
     if (article == NULL) {
 	oldsize = artsize;
-	article = NEW(char, oldsize + 1 + 1);
+	article = xmalloc(oldsize + 1 + 1);
     }
     else if (artsize > oldsize) {
 	oldsize = artsize;
-	RENEW(article, char, oldsize + 1 + 1);
+        article = xrealloc(article, oldsize + 1 + 1);
     }
 
     /* Read in the article. */
     for (p = article, left = artsize; left; p += i, left -= i)
 	if ((i = read(fd, p, left)) <= 0) {
 	    i = errno;
-	    syslog(L_ERROR, "cant read wanted %d got %d %m",
-		artsize, artsize - left);
+            warn("cannot read, wanted %d got %d", artsize, artsize - left);
 #if	0
 	    /* Don't do this -- if the article gets re-processed we
 	     * will end up accepting the truncated version. */
@@ -434,7 +439,7 @@ ReadBytecount(fd, artsize)
 	    article[artsize] = '\0';
 	    Reject(article, "short read (%s?)", strerror(i));
 #endif	/* 0 */
-	    return TRUE;
+	    return true;
 	}
     if (p[-1] != '\n')
 	*p++ = '\n';
@@ -448,11 +453,8 @@ ReadBytecount(fd, artsize)
 /*
 **  Read a single text line; not unlike fgets().  Just more inefficient.
 */
-STATIC BOOL
-ReadLine(p, size, fd)
-    char	*p;
-    int		size;
-    int		fd;
+static bool
+ReadLine(char *p, int size, int fd)
 {
     char	*save;
 
@@ -460,66 +462,63 @@ ReadLine(p, size, fd)
     for (save = p; size > 0; p++, size--) {
 	if (read(fd, p, 1) != 1) {
 	    *p = '\0';
-	    syslog(L_FATAL, "cant read first line got %s %m", save);
-	    exit(1);
+            sysdie("cannot read first line, got %s", save);
 	}
 	if (*p == '\n') {
 	    *p = '\0';
-	    return TRUE;
+	    return true;
 	}
     }
     *p = '\0';
-    syslog(L_FATAL, "bad_line too long %s", save);
-    return FALSE;
+    warn("bad_line too long %s", save);
+    return false;
 }
 
 
 /*
 **  Unpack a single batch.
 */
-STATIC BOOL
-UnpackOne(fdp, countp)
-    int		*fdp;
-    int		*countp;
+static bool
+UnpackOne(int *fdp, size_t *countp)
 {
 #if	defined(DO_RNEWSPROGS)
     char	path[(SMBUF * 2) + 1];
     char	*p;
 #endif	/* defined(DO_RNEWSPROGS) */
     char	buff[SMBUF];
-    STRING	cargv[4];
+    const char *cargv[4];
     int		artsize;
     int		i;
     int		gzip = 0;
-    BOOL	HadCount;
-    BOOL	SawCunbatch;
+    bool	HadCount;
+    bool	SawCunbatch;
     int		len;
 
     *countp = 0;
-    for (SawCunbatch = FALSE, HadCount = FALSE; ; ) {
+    for (SawCunbatch = false, HadCount = false; ; ) {
 	/* Get the first character. */
 	if ((i = read(*fdp, &buff[0], 1)) < 0) {
-	    syslog(L_ERROR, "cant read first character %m");
-	    return FALSE;
+            syswarn("cannot read first character");
+	    return false;
 	}
 	if (i == 0)
 	    break;
 
 	if (buff[0] == 0x1f)
 	    gzip = 1;
-	else if (buff[0] != RNEWS_MAGIC1)
+	else if (buff[0] != '#')
 	    /* Not a batch file.  If we already got one count, the batch
 	     * is corrupted, else read rest of input as an article. */
-	    return HadCount ? FALSE : ReadRemainder(*fdp, buff[0], '\0');
+	    return HadCount ? false : ReadRemainder(*fdp, buff[0], '\0');
 
 	/* Get the second character. */
 	if ((i = read(*fdp, &buff[1], 1)) < 0) {
-	    syslog(L_ERROR, "cant read second character %m");
-	    return FALSE;
+            syswarn("cannot read second character");
+	    return false;
 	}
 	if (i == 0)
 	    /* A one-byte batch? */
-	    return FALSE;
+	    return false;
 
 	/* Check second magic character. */
 	/* gzipped ($1f$8b) or compressed ($1f$9d) */
@@ -527,50 +526,50 @@ UnpackOne(fdp, countp)
 	    cargv[0] = "gzip";
 	    cargv[1] = "-d";
 	    cargv[2] = NULL;
-	    lseek(*fdp, (OFFSET_T) 0, 0); /* Back to the beginning */
+	    lseek(*fdp, 0, 0); /* Back to the beginning */
 	    *fdp = StartChild(*fdp, _PATH_GZIP, cargv);
 	    if (*fdp < 0)
-	        return FALSE;
+	        return false;
 	    (*countp)++;
-	    SawCunbatch = TRUE;
+	    SawCunbatch = true;
 	    continue;
 	}
-	if (buff[1] != RNEWS_MAGIC2)
-	    return HadCount ? FALSE : ReadRemainder(*fdp, buff[0], buff[1]);
+	if (buff[1] != '!')
+	    return HadCount ? false : ReadRemainder(*fdp, buff[0], buff[1]);
 
 	/* Some kind of batch -- get the command. */
 	if (!ReadLine(&buff[2], (int)(sizeof buff - 3), *fdp))
-	    return FALSE;
+	    return false;
 
 	if (strncmp(buff, "#! rnews ", 9) == 0) {
 	    artsize = atoi(&buff[9]);
 	    if (artsize <= 0) {
-		syslog(L_ERROR, "bad_line bad count %s", buff);
-		return FALSE;
+                syswarn("bad_line bad count %s", buff);
+		return false;
 	    }
-	    HadCount = TRUE;
+	    HadCount = true;
 	    if (ReadBytecount(*fdp, artsize))
 		continue;
-	    return FALSE;
+	    return false;
 	}
 
 	if (HadCount)
 	    /* Already saw a bytecount -- probably corrupted. */
-	    return FALSE;
+	    return false;
 
 	if (strcmp(buff, "#! cunbatch") == 0) {
 	    if (SawCunbatch) {
-		syslog(L_ERROR, "nested_cunbatch");
-		return FALSE;
+                syswarn("nested_cunbatch");
+		return false;
 	    }
 	    cargv[0] = UNPACK;
 	    cargv[1] = "-d";
 	    cargv[2] = NULL;
 	    *fdp = StartChild(*fdp, _PATH_GZIP, cargv);
 	    if (*fdp < 0)
-		return FALSE;
+		return false;
 	    (*countp)++;
-	    SawCunbatch = TRUE;
+	    SawCunbatch = true;
 	    continue;
 	}
 
@@ -583,11 +582,11 @@ UnpackOne(fdp, countp)
 	else
 	    p = &buff[3];
 	if (strchr(_PATH_RNEWSPROGS, '/') == NULL) {
-	    (void)sprintf(path, "%s/%s/%s", innconf->pathbin,
-					_PATH_RNEWSPROGS, p);
+	    snprintf(path, sizeof(path), "%s/%s/%s", innconf->pathbin,
+                     _PATH_RNEWSPROGS, p);
 	    len = strlen(innconf->pathbin) + 1 + sizeof _PATH_RNEWSPROGS;
 	} else {
-	    (void)sprintf(path, "%s/%s", _PATH_RNEWSPROGS, p);
+	    snprintf(path, sizeof(path), "%s/%s", _PATH_RNEWSPROGS, p);
 	    len = sizeof _PATH_RNEWSPROGS;
 	}
 	for (p = &path[len]; *p; p++)
@@ -597,15 +596,15 @@ UnpackOne(fdp, countp)
 	    }
 	*fdp = StartChild(*fdp, path, cargv);
 	if (*fdp < 0)
-	    return FALSE;
+	    return false;
 	(*countp)++;
 	continue;
 #else
-	syslog(L_ERROR, "bad_format unknown command %s", buff);
-	return FALSE;
+        warn("bad_format unknown command %s", buff);
+	return false;
 #endif	/* defined(DO_RNEWSPROGS) */
     }
-    return TRUE;
+    return true;
 }
 
 
@@ -614,29 +613,26 @@ UnpackOne(fdp, countp)
 **  errors with xperror as well as syslog, since we're probably being run
 **  interactively.
 */
-STATIC void
-Unspool()
+static void
+Unspool(void)
 {
-    register DIR	*dp;
+    DIR	*dp;
     struct dirent       *ep;
-    register BOOL	ok;
+    bool	ok;
     struct stat		Sb;
-    char		buff[SMBUF];
     char		hostname[10];
-    int			fd;
-    int			i;
+    int			fd, fd2;
+    size_t		i;
+    char                *badname, *uuhost;
+
+    message_handlers_die(2, message_log_stderr, message_log_syslog_err);
+    message_handlers_warn(2, message_log_stderr, message_log_syslog_err);
 
     /* Go to the spool directory, get ready to scan it. */
-    if (chdir(innconf->pathincoming) < 0) {
-	xperror(innconf->pathincoming);
-	syslog(L_FATAL, "cant cd %s %m", innconf->pathincoming);
-	exit(1);
-    }
-    if ((dp = opendir(".")) == NULL) {
-	xperror("Can't open spool directory");
-	syslog(L_FATAL, "cant opendir . %m");
-	exit(1);
-    }
+    if (chdir(innconf->pathincoming) < 0)
+        sysdie("cannot chdir to %s", innconf->pathincoming);
+    if ((dp = opendir(".")) == NULL)
+        sysdie("cannot open spool directory");
 
     /* Loop over all files, and parse them. */
     while ((ep = readdir(dp)) != NULL) {
@@ -644,50 +640,62 @@ Unspool()
 	if (InputFile[0] == '.')
 	    continue;
 	if (stat(InputFile, &Sb) < 0 && errno != ENOENT) {
-	    xperror(InputFile);
-	    syslog(L_ERROR, "cant stat %s %m", InputFile);
+            syswarn("cannot stat %s", InputFile);
 	    continue;
 	}
 
 	if (!S_ISREG(Sb.st_mode))
 	    continue;
 
-	if ((fd = open(InputFile, O_RDONLY)) < 0) {
-	    if (errno != ENOENT) {
-		xperror(InputFile);
-		syslog(L_ERROR, "cant open %s %m", InputFile);
-	    }
+	if ((fd = open(InputFile, O_RDWR)) < 0) {
+	    if (errno != ENOENT)
+                syswarn("cannot open %s", InputFile);
 	    continue;
 	}
+
+	/* Make sure multiple Unspools don't stomp on eachother. */
+	if (!inn_lock_file(fd, INN_LOCK_WRITE, 0)) {
+	    close(fd);
+	    continue;
+	}
+
 	/* Get UUCP host from spool file, deleting the mktemp XXXXXX suffix. */
-	if ((i = strlen(InputFile)) > 6)
+	uuhost = UUCPHost;
+	hostname[0] = 0;
+	if ((i = strlen(InputFile)) > 6) {
 	    i -= 6;
-	if (i > sizeof hostname - 1)
-	    /* Just in case someone wrote their own spooled file. */
-	    i = sizeof hostname - 1;
-	(void)strncpy(hostname, InputFile, i);
-	hostname[i] = '\0';
-	UUCPHost = hostname;
+	    if (i > sizeof hostname - 1)
+		/* Just in case someone wrote their own spooled file. */
+		i = sizeof hostname - 1;
+	    strlcpy(hostname, InputFile, i + 1);
+	    UUCPHost = hostname;
+	}
 	ok = UnpackOne(&fd, &i);
-	(void)close(fd);
 	WaitForChildren(i);
+	UUCPHost = uuhost;
 
 	if (!ok) {
-	    TempName(PathBadNews, buff);
-	    (void)fprintf(stderr, "Unspooling failed saving to %s\n", buff);
-	    syslog(L_ERROR, "cant unspool saving to %s", buff);
-	    if (rename(InputFile, buff) < 0) {
-		xperror(buff);
-		syslog(L_FATAL, "cant rename %s to %s %m", InputFile, buff);
-		exit(1);
-	    }
+            badname = concat(PathBadNews, "/", hostname, "XXXXXX", (char *) 0);
+            fd2 = mkstemp(badname);
+            if (fd2 < 0)
+                sysdie("cannot create temporary file");
+            close(fd2);
+            warn("cant unspool saving to %s", badname);
+	    if (rename(InputFile, badname) < 0)
+                sysdie("cannot rename %s to %s", InputFile, badname);
+	    close(fd);
+            free(badname);
 	    continue;
 	}
 
 	if (unlink(InputFile) < 0)
-	    syslog(L_ERROR, "cant remove %s %m", InputFile);
+            syswarn("cannot remove %s", InputFile);
+	close(fd);
     }
-    (void)closedir(dp);
+    closedir(dp);
+
+    message_handlers_die(1, message_log_syslog_err);
+    message_handlers_warn(1, message_log_syslog_err);
 }
 
 
@@ -697,48 +705,39 @@ Unspool()
 **  we can do if this routine fails, unfortunately.  Perhaps try to use
 **  an alternate filesystem?
 */
-STATIC void
-Spool(fd, mode)
-    register int	fd;
-    int			mode;
+static void
+Spool(int fd, int mode)
 {
-    register int	spfd;
-    register int	i;
-    register int	j;
-    register char	*p;
-    char		temp[BUFSIZ];
-    char		buff[BUFSIZ];
-    int			count;
-    int			status;
+    int spfd;
+    int i;
+    int j;
+    char *tmpspool, *spoolfile, *p;
+    char buff[BUFSIZ];
+    int count;
+    int status;
 
-    if(mode == 'N')
+    if (mode == 'N')
 	exit(9);
-    TempName(innconf->pathincoming, buff);
-    p = strrchr(buff, '/');
-    if (p == NULL)
-	exit(10);
-    *p = '\0';
-    strcpy(temp, buff);
-    strcat(temp, "/.");
-    strcat(temp, ++p);
-    (void)umask(0);
-    if ((spfd = open(temp, O_WRONLY | O_CREAT, BATCHFILE_MODE)) < 0) {
-	syslog(L_FATAL, "cant open %s %m", temp);
-	exit(1);
-    }
+    tmpspool = concat(innconf->pathincoming, "/.",
+		UUCPHost ? UUCPHost : "", "XXXXXX", (char *)0);
+    spfd = mkstemp(tmpspool);
+    if (spfd < 0)
+        sysdie("cannot create temporary batch file %s", tmpspool);
+    if (fchmod(spfd, BATCHFILE_MODE) < 0)
+        sysdie("cannot chmod temporary batch file %s", tmpspool);
 
     /* Read until we there is nothing left. */
     for (status = 0, count = 0; (i = read(fd, buff, sizeof buff)) != 0; ) {
 	/* Break out on error. */
 	if (i < 0) {
-	    syslog(L_FATAL, "cant read after %d %m", count);
+            syswarn("cannot read after %d", count);
 	    status++;
 	    break;
 	}
 	/* Write out what we read. */
 	for (count += i, p = buff; i; p += j, i -= j)
-	    if ((j = write(spfd, (POINTER)p, (SIZE_T)i)) <= 0) {
-		syslog(L_FATAL, "cant write around %d %m", count);
+	    if ((j = write(spfd, p, i)) <= 0) {
+                syswarn("cannot write around %d", count);
 		status++;
 		break;
 	    }
@@ -746,16 +745,26 @@ Spool(fd, mode)
 
     /* Close the file. */
     if (close(spfd) < 0) {
-	syslog(L_FATAL, "cant close spooled rnews %m");
+        syswarn("cannot close spooled article %s", tmpspool);
 	status++;
     }
 
     /* Move temp file into the spool area, and exit appropriately. */
-    TempName(innconf->pathincoming, buff);
-    if (rename(temp, buff) < 0) {
-	syslog(L_FATAL, "cant rename %s to %s %m", temp, buff);
-	status++;
+    spoolfile = concat(innconf->pathincoming, "/",
+		UUCPHost ? UUCPHost : "", "XXXXXX", (char *)0);
+    spfd = mkstemp(spoolfile);
+    if (spfd < 0) {
+        syswarn("cannot create spool file %s", spoolfile);
+        status++;
+    } else {
+        close(spfd);
+        if (rename(tmpspool, spoolfile) < 0) {
+            syswarn("cannot rename %s to %s", tmpspool, spoolfile);
+            status++;
+        }
     }
+    free(tmpspool);
+    free(spoolfile);
     exit(status);
     /* NOTREACHED */
 }
@@ -765,7 +774,7 @@ Spool(fd, mode)
 **  Try to read the password file and open a connection to a remote
 **  NNTP server.
 */
-STATIC BOOL OpenRemote(char *server, int port, char *buff)
+static bool OpenRemote(char *server, int port, char *buff)
 {
     int		i;
 
@@ -775,84 +784,82 @@ STATIC BOOL OpenRemote(char *server, int port, char *buff)
     else
 	i = NNTPremoteopen(port, &FromServer, &ToServer, buff);
     if (i < 0)
-	return FALSE;
+	return false;
 
     *buff = '\0';
     if (NNTPsendpassword(server, FromServer, ToServer) < 0) {
 	int oerrno = errno;
-	(void)fclose(FromServer);
-	(void)fclose(ToServer);
+	fclose(FromServer);
+	fclose(ToServer);
 	errno = oerrno;
-	return FALSE;
+	return false;
     }
-    return TRUE;
+    return true;
 }
 
 
 /*
 **  Can't connect to server; print message and spool if necessary.
 */
-STATIC NORETURN
-CantConnect(buff, mode, fd)
-    char	*buff;
-    int		mode;
-    int		fd;
+static void
+CantConnect(char *buff, int mode, int fd)
 {
     if (buff[0])
-	syslog(L_NOTICE, "rejected connection %s", REMclean(buff));
+        notice("rejected connection %s", REMclean(buff));
     else
-	syslog(L_FATAL, "cant open_remote %m");
+        syswarn("cant open_remote");
     if (mode != 'U')
 	Spool(fd, mode);
     exit(1);
 }
 
 
-/*
-**  Log an incorrect usage.
-*/
-STATIC NORETURN
-Usage()
-{
-    syslog(L_FATAL, "usage error");
-    exit(1);
-}
-
-
 int main(int ac, char *av[])
 {
     int		fd;
     int		i;
+    size_t	count;
     int		mode;
     char	buff[SMBUF];
     int         port = NNTP_PORT;
 
     /* First thing, set up logging and our identity. */
     openlog("rnews", L_OPENLOG_FLAGS, LOG_INN_PROG);
-#if !defined(__FreeBSD__) && !defined(__bsdi__) && !defined (__NetBSD__)
-    if (setgid(getegid()) < 0) {
-	syslog(L_FATAL, "cant setgid to %d %m", getegid());
-	exit(1);
-    }
-    if (setuid(geteuid()) < 0) {
-	syslog(L_FATAL, "cant setuid to %d %m", geteuid());
-	exit(1);
-    }
-#endif
-     if (ReadInnConf() < 0) exit(1);
-     UUCPHost = getenv(_ENV_UUCPHOST);
-     PathBadNews = COPY(cpcatpath(innconf->pathincoming, _PATH_BADNEWS));
-     port = innconf->nnrpdpostport;
+    message_program_name = "rnews";
+    message_handlers_notice(1, message_log_syslog_notice);
+    message_handlers_warn(1, message_log_syslog_err);
+    message_handlers_die(1, message_log_syslog_err);
 
-    (void)umask(NEWSUMASK);
+    /* Make sure that we switch to the news user if we're running as root,
+       since we may spool files and don't want those files owned by root.
+       Don't require that we be running as the news user, though; there are
+       other setups where rnews might be setuid news or be run by other
+       processes in the news group. */
+    if (getuid() == 0 || geteuid() == 0) {
+        struct passwd *pwd;
+
+        pwd = getpwnam(NEWSUSER);
+        if (pwd == NULL)
+            die("can't resolve %s to a UID (account doesn't exist?)",
+                NEWSUSER);
+        setuid(pwd->pw_uid);
+    }
+
+    if (!innconf_read(NULL))
+        exit(1);
+    UUCPHost = getenv(_ENV_UUCPHOST);
+    PathBadNews = concatpath(innconf->pathincoming, _PATH_BADNEWS);
+    port = innconf->nnrpdpostport;
+
+    umask(NEWSUMASK);
 
     /* Parse JCL. */
-    fd = STDIN;
+    fd = STDIN_FILENO;
     mode = '\0';
     while ((i = getopt(ac, av, "h:P:NUvr:S:")) != EOF)
 	switch (i) {
 	default:
-	    Usage();
+	    die("usage error");
 	    /* NOTRTEACHED */
 	case 'h':
 	    UUCPHost = *optarg ? optarg : NULL;
@@ -865,7 +872,7 @@ int main(int ac, char *av[])
 	    port = atoi(optarg);
 	    break;
 	case 'v':
-	    Verbose = TRUE;
+	    Verbose = true;
 	    break;
 	case 'r':
 	case 'S':
@@ -878,17 +885,15 @@ int main(int ac, char *av[])
     /* Parse arguments.  At most one, the input file. */
     switch (ac) {
     default:
-	Usage();
+        die("usage error");
 	/* NOTREACHED */
     case 0:
 	break;
     case 1:
 	if (mode == 'U')
-	    Usage();
-	if (freopen(av[0], "r", stdin) == NULL) {
-	    syslog(L_FATAL, "cant freopen %s %m", av[0]);
-	    exit(1);
-	}
+            die("usage error");
+	if (freopen(av[0], "r", stdin) == NULL)
+            sysdie("cannot freopen %s", av[0]);
 	fd = fileno(stdin);
 	InputFile = av[0];
 	break;
@@ -919,22 +924,25 @@ int main(int ac, char *av[])
 		CantConnect(buff, mode, fd);
 #endif	/* defined(DO_RNEWSLOCALCONNECT) */
     }
-    CloseOnExec((int)fileno(FromServer), TRUE);
-    CloseOnExec((int)fileno(ToServer), TRUE);
+    close_on_exec(fileno(FromServer), true);
+    close_on_exec(fileno(ToServer), true);
 
     /* Execute the command. */
     if (mode == 'U')
 	Unspool();
     else {
-	if (!UnpackOne(&fd, &i))
+	if (!UnpackOne(&fd, &count)) {
+	    lseek(fd, 0, 0);
 	    Spool(fd, mode);
-	WaitForChildren(i);
+	}
+	close(fd);
+	WaitForChildren(count);
     }
 
     /* Tell the server we're quitting, get his okay message. */
-    (void)fprintf(ToServer, "quit\r\n");
-    (void)fflush(ToServer);
-    (void)fgets(buff, sizeof buff, FromServer);
+    fprintf(ToServer, "quit\r\n");
+    fflush(ToServer);
+    fgets(buff, sizeof buff, FromServer);
 
     /* Return the appropriate status. */
     exit(0);
