@@ -1,66 +1,26 @@
-/* -*- c -*-
- *
- * Author:      James Brister <brister@vix.com> -- berkeley-unix --
- * Start Date:  Wed Dec 27 17:10:18 1995
- * Project:     INN (innfeed)
- * File:        misc.c
- * RCSId:       $Id: misc.c,v 1.8.2.7 2002/08/24 00:23:07 rra Exp $
- *
- * Copyright:   Copyright (c) 1996 by Internet Software Consortium
- *
- *              Permission to use, copy, modify, and distribute this
- *              software for any purpose with or without fee is hereby
- *              granted, provided that the above copyright notice and this
- *              permission notice appear in all copies.
- *
- *              THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE
- *              CONSORTIUM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
- *              SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
- *              MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL INTERNET
- *              SOFTWARE CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT,
- *              INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- *              WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- *              WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- *              TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
- *              USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * Description: 
- * 
- */
-
-#if ! defined (lint)
-static const char *rcsid = "$Id: misc.c,v 1.8.2.7 2002/08/24 00:23:07 rra Exp $" ;
-static void use_rcsid (const char *rid) {   /* Never called */
-  use_rcsid (rcsid) ; use_rcsid (rid) ;
-}
-#endif
+/*  $Id: misc.c,v 1.29.2.1 2004/01/25 20:07:10 rra Exp $
+**
+**  Helper routines for the innfeed program.
+**
+**  Written by James Brister <brister@vix.com>
+*/
 
 #include "innfeed.h"
 #include "config.h"
 #include "clibrary.h"
 
-#include <arpa/nameser.h>
 #include <assert.h>
 #include <ctype.h>
 #include <errno.h>
+#include <fcntl.h>
 #include <netdb.h>
 #include <netinet/in.h>
-#include <resolv.h>
 #include <signal.h>
-#include <stdarg.h>
 #include <syslog.h>
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <time.h>
 
-#ifdef HAVE_FCNTL_H
-# include <fcntl.h>
-#endif
-
-#if defined (HAVE_UNISTD_H)
-# include <unistd.h>
-#endif
-
 /* FIXME: Default to a max length of 256 characters for path names if the
    host headers doesn't give better information.  Should be replaced by the
    code from Stevens. */
@@ -68,33 +28,92 @@ static void use_rcsid (const char *rid) {   /* Never called */
 # define PATH_MAX 256
 #endif
 
+#include "inn/messages.h"
 #include "libinn.h"
 
 #include "endpoint.h"
 #include "misc.h"
-#include "msgs.h"
 #include "tape.h"
 
-u_int openfds ;
-char *program ;                 /* this should be set to argv[0] */
+unsigned int openfds ;
 int debuggingOutput ;
-u_int loggingLevel ;
+unsigned int loggingLevel ;
 char **PointersFreedOnExit ;
-static void dolog (int level, const char *fmt, va_list args) ;
+
+bool debuggingDump = true ;
+extern void (*gPrintInfo) (void) ;
+void (*gCleanUp) (void) = 0 ;
 
 
-
-
-
-int maxFds (void)
+/* Log a message to stderr, called from warn or die.  Mostly the same as the
+   standard message_log_stderr, but prepends the date to each line. */
+void
+error_log_stderr_date(int len UNUSED, const char *fmt, va_list args, int err)
 {
-  static int size = 0 ;
-  size = getfdcount () ;
-  return size;
+    char timebuff[30];
+    time_t now;
+    struct tm *tm;
+
+    now = time(NULL);
+    tm = localtime(&now);
+    strftime(timebuff, sizeof(timebuff), "%Y-%m-%d %H:%M:%S", tm);
+    fprintf(stderr, "%s %s: ", timebuff,
+            (message_program_name ? message_program_name : "UNKNOWN"));
+    vfprintf(stderr, fmt, args);
+    if (err) fprintf(stderr, ": %s", strerror(err));
+    fprintf(stderr, "\n");
+}
+
+/* If desired, print out the state of innfeed, call a cleanup function, and
+   then dump core.  Used as an exit handler for die. */
+int
+dump_core(void)
+{
+#if SNAPSHOT_ON_DIE
+    if (debuggingDump && gPrintInfo != NULL)
+        (*gPrintInfo)();
+#endif
+
+    if (gCleanUp != NULL)
+        (*gCleanUp)();
+  
+    if (CORE_DIRECTORY != NULL)
+        chdir(CORE_DIRECTORY);
+    else
+        chdir(getTapeDirectory());
+  
+    sleep(5);
+    abort();
+
+    /* Not reached. */
+    return 1;
+}
+
+/* An alternate version of die, used when we don't want to dump core.  This
+   should somehow eventually be phased out to simplify things; it's
+   basically a copy of die() from lib/error.c that ignores the cleanup
+   handler and has innfeed's handlers hard-coded (ugh). */
+void
+logAndExit(int status, const char *format, ...)
+{
+    va_list args;
+    int length;
+
+    va_start(args, format);
+    length = vsnprintf(NULL, 0, format, args);
+    va_end(args);
+    va_start(args, format);
+    error_log_stderr_date(length, format, args, 0);
+    va_end(args);
+    va_start(args, format);
+    message_log_syslog_err(length, format, args, 0);
+    va_end(args);
+    exit(status);
 }
 
 
-void d_printf (u_int level, const char *fmt, ...) 
+
+void d_printf (unsigned int level, const char *fmt, ...) 
 {
   static pid_t myPid ;
   char timeString [30] ;
@@ -108,41 +127,18 @@ void d_printf (u_int level, const char *fmt, ...)
     return ;
   
   now = theTime() ;
-  strcpy (timeString, ctime (&now) + 4) ; /* strip off leading day name */
+  /* strip off leading day name */
+  strlcpy (timeString, ctime (&now) + 4, sizeof (timeString)) ;
   timeString [15] = '\0' ;      /* strip off trailing year and newline */
 
   va_start (ap, fmt) ;
   fprintf (stderr, "%s %s[%ld]: ",timeString,
-           (program ? program : "UNKNOWN PROGRAM NAME"), (long) myPid) ;
+           (message_program_name ? message_program_name : "UNKNOWN"),
+           (long) myPid) ;
   vfprintf (stderr, fmt, ap) ;
   va_end (ap) ;
 }
 
-bool debuggingDump = true ;
-extern void (*gPrintInfo) (void) ;
-void (*gCleanUp) (void) = 0 ;
-
-static void dolog (int level, const char *fmt, va_list args)
-{
-  time_t now = time (NULL) ;
-  char timeString [30] ;
-  char *p = NULL ;
-  int out ;
-  
-  strcpy (timeString,ctime (&now)) ;
-  timeString [24] = '\0' ;
-  
-  fprintf (stderr, "%s %s: ",
-           timeString, (program ? program : "UNKNOWN PROGRAM NAME")) ;
-  out = vfprintf (stderr, fmt, args) ;
-  fprintf (stderr,"\n") ;
-
-  p = malloc (out + 10) ;
-  vsprintf (p,fmt,args) ;
-  syslog (level,"%s",p) ;
-  free (p) ;
-}
-
 void logOrPrint (int level, FILE *fp, const char *fmt, ...)
 {
   va_list ap ;
@@ -155,59 +151,14 @@ void logOrPrint (int level, FILE *fp, const char *fmt, ...)
     }
   else
     {
-      char buffer [512] ;
+      char buffer [512] ;      /* gag me */
 
-      vsprintf (buffer,fmt,ap) ;
+      vsnprintf (buffer,sizeof (buffer),fmt,ap) ;
       syslog (level,buffer) ;
     }
   va_end (ap) ;
 }
 
-void die (const char *fmt, ...)
-{
-  va_list ap ;
-
-  va_start (ap, fmt) ;
-  dolog (LOG_ERR,fmt,ap) ;
-  va_end (ap) ;
-
-#if SNAPSHOT_ON_DIE
-  if (debuggingDump && gPrintInfo != NULL)
-    gPrintInfo () ;
-#endif
-
-  if (gCleanUp != NULL)
-    gCleanUp () ;
-  
-  if (CORE_DIRECTORY != NULL)
-    (void) chdir (CORE_DIRECTORY) ;
-  else
-    (void) chdir (getTapeDirectory()) ;
-  
-  sleep (5) ;
-  abort () ;
-}
-
-void warn (const char *fmt, ...)
-{
-  va_list ap ;
-
-  va_start (ap, fmt) ;
-  dolog (LOG_WARNING,fmt,ap) ;
-  va_end (ap) ;
-}
-
-
-void logAndExit (int exitVal, const char *fmt, ...)
-{
-  va_list ap ;
-
-  va_start (ap,fmt) ;
-  dolog (LOG_CRIT,fmt,ap) ;
-  va_end (ap) ;
-
-  exit (exitVal) ;
-}
 
 
 /* return true if the file exists and is a regular file. */
@@ -247,10 +198,10 @@ bool getNntpResponse (char *p, int *code, char **rest)
   if (p == NULL)
     return false ;
   
-  while (*p && isspace (*p))
+  while (*p && CTYPE (isspace, *p))
     p++ ;
 
-  while (*p && isdigit (*p))
+  while (*p && CTYPE (isdigit, *p))
     {
       digits++ ;
       cd = (cd * 10) + (*p - '0') ;
@@ -263,7 +214,7 @@ bool getNntpResponse (char *p, int *code, char **rest)
   if (*p == '-')
     p++ ;
   
-  while (*p && isspace (*p))
+  while (*p && CTYPE (isspace, *p))
     p++ ;
       
   if (rest)
@@ -275,25 +226,6 @@ bool getNntpResponse (char *p, int *code, char **rest)
 }
 
 
-#if 0
-/* pinched from INN main source */
-int
-writev(fd, vp, vpcount)
-    int                 fd;
-    struct iovec        *vp;
-    int                 vpcount;
-{
-    int                 count;
-
-    for (count = 0; --vpcount >= 0; count += vp->iov_len, vp++)
-        if (xwrite(fd, vp->iov_base, vp->iov_len) < 0)
-            return -1;
-    return count;
-}
-
-#endif
-
-
 
 /* Pull out a message id from a response on to a streaming command */
 char *getMsgId (const char *p)
@@ -301,23 +233,19 @@ char *getMsgId (const char *p)
   const char *q ;
   char *rval ;
   
-  while (*p && isspace (*p)) p++ ;
-  while (*p && !isspace (*p)) p++ ; /* skip response code */
-  while (*p && isspace (*p)) p++ ;
+  while (*p && CTYPE (isspace, *p)) p++ ;
+  while (*p && !CTYPE (isspace, *p)) p++ ; /* skip response code */
+  while (*p && CTYPE (isspace, *p)) p++ ;
 
   if ( *p == '\0' )
     return NULL ;
 
   q = p ;
-  while ( *q && !isspace (*q) )
+  while ( *q && !CTYPE (isspace, *q) )
     q++ ;
-  
-  rval = MALLOC ((size_t) (q - p + 1)) ;
-  assert (rval != NULL) ;
 
-  strncpy (rval,p,(size_t) (q - p)) ;
-  rval [q - p] = '\0' ;
-  
+  rval = xstrndup (p, q - p) ;
+
   return rval ;
 }
 
@@ -328,12 +256,12 @@ char *findNonBlankString (char *ptr, char **tail)
 {
   char *p, *q ;
 
-  for (p = ptr ; *p && isspace (*p) ; p++)
+  for (p = ptr ; *p && CTYPE (isspace, *p) ; p++)
     /* nada */ ;
   if ( ! *p )
     return NULL ;
 
-  for (q = p ; *q && !isspace (*q) ; q++)
+  for (q = p ; *q && !CTYPE (isspace, *q) ; q++)
     /* nada */ ;
 
   *tail = q ;
@@ -393,7 +321,7 @@ char *mystrtok (char *line, const char *sep)
 void trim_ws (char *string)
 {
   char *p ;
-  u_int len ;
+  unsigned int len ;
 
   assert (string != NULL) ;
 
@@ -401,7 +329,7 @@ void trim_ws (char *string)
   if (len == 0)
     return ;
   
-  for (p = string + len - 1 ; p >= string && isspace (*p) ; p--)
+  for (p = string + len - 1 ; p >= string && CTYPE (isspace, *p) ; p--)
     /* nada */ ;
   *++p = '\0' ;
 }
@@ -463,15 +391,16 @@ bool lockFile (const char *fileName)
   int fd, i ;
   pid_t pid = getpid () ;
 
-  strcpy (realName,fileName) ;
+  strlcpy (realName,fileName,sizeof (realName)) ;
   if ((p = strrchr (realName, '/')) != NULL)
     {
       *p = '\0' ;
-      sprintf (tmpName, "%s/lockf%ld", realName, (long) pid) ;
+      snprintf (tmpName, sizeof(tmpName), "%s/lockf%ld", realName,
+                (long) pid) ;
       *p = '/' ;
     }
   else
-    sprintf (tmpName, "lockf%ld", (long) pid) ;
+    snprintf (tmpName, sizeof(tmpName), "lockf%ld", (long) pid) ;
   
   /* Create the temporary name for the lock file. */
   while ((fd = open (tmpName, O_RDWR | O_CREAT | O_EXCL, 0644)) < 0)
@@ -480,13 +409,13 @@ bool lockFile (const char *fileName)
         {
           default:
             unlink (tmpName) ;
-            syslog (LOG_ERR,NO_OPEN_LOCK,tmpName) ;
+            syswarn ("ME lock file open: %s", tmpName) ;
             return false ;
 
           case EEXIST:
             if (unlink (tmpName) < 0)
               {
-                syslog (LOG_ERR,NO_UNLINK_LOCK,tmpName) ;
+                syswarn ("ME lock file unlink: %s", tmpName) ;
                 return false ;
               }
             break;
@@ -494,10 +423,10 @@ bool lockFile (const char *fileName)
     }
 
   /* stick our pid in the temp file. */
-  sprintf (buff,"%ld\n",(long) pid) ;
+  snprintf (buff,sizeof(buff),"%ld\n",(long) pid) ;
   if (write (fd,buff,(size_t) strlen (buff)) != (int) strlen (buff))
     {
-      syslog (LOG_ERR,NO_WRITE_LOCK_PID) ;
+      syswarn ("ME lock file pid-write") ;
       close (fd) ;
       unlink (tmpName) ;
       return false ;
@@ -510,7 +439,7 @@ bool lockFile (const char *fileName)
       switch (errno) 
         {
           default:              /* opps. bailing out. */
-            syslog (LOG_ERR,NO_LINK_LOCK,realName) ;
+            syswarn ("ME lock file link: %s", realName) ;
             unlink (tmpName) ;
             return false ;
 
@@ -519,7 +448,7 @@ bool lockFile (const char *fileName)
                see if that process is still alive. */
             if ((fd = open (realName,O_RDONLY)) < 0)
               {
-                syslog (LOG_ERR,NO_OPEN_LOCK,realName) ;
+                syswarn ("ME lock file open: %s", realName) ;
                 unlink (tmpName) ;
                 return false ;
               }
@@ -536,7 +465,7 @@ bool lockFile (const char *fileName)
             pid = (pid_t) atol (buff) ;
             if (pid <= 0)
               {
-                syslog (LOG_ERR,BAD_PID,realName,buff) ;
+                warn ("ME lock bad-pid info in %s: %s", realName, buff) ;
                 unlink (tmpName) ;
                 return false ;
               }
@@ -545,7 +474,8 @@ bool lockFile (const char *fileName)
                it's still alive. */
             if (kill (pid,0) == 0)
               {
-                syslog (LOG_ERR,LOCK_EXISTS,realName,(int) pid) ;
+                warn ("ME lock in-use already: %s by pid %ld", realName,
+                      (unsigned long) pid);
                 unlink (tmpName) ;
                 return false ;    /* process is still alive */
               }
@@ -553,7 +483,7 @@ bool lockFile (const char *fileName)
             /* process that took out the lock is gone */
             if (unlink (realName) < 0)
               {
-                syslog (LOG_ERR,NO_UNLINK_LOCK,realName) ;
+                syswarn ("ME lock file unlink: %s", realName) ;
                 unlink (tmpName) ;
                 return false ;
               }
@@ -568,7 +498,7 @@ bool lockFile (const char *fileName)
 
 void unlockFile (const char *lockfile)
 {
-  (void) unlink (lockfile) ;
+  unlink (lockfile) ;
 }
 
 
@@ -642,7 +572,7 @@ bool isOlder (const char *file1, const char *file2)
 
 void freeCharP (char *charp)
 {
-  FREE (charp) ;
+  free (charp) ;
 }
 
 
@@ -671,16 +601,14 @@ void addPointerFreedOnExit (char *pointerToFree)
 
   if (nextPointer == 0 || nextPointer == totalPointers - 1)
     {
-      register int i;
+      int i;
 
       totalPointers += 16 ;
       if (PointersFreedOnExit == NULL)
-	PointersFreedOnExit = ALLOC (char *, totalPointers) ;
+	PointersFreedOnExit = xmalloc (sizeof(char *) * totalPointers) ;
       else
 	PointersFreedOnExit =
-	  REALLOC (PointersFreedOnExit, char *, totalPointers) ;
-
-      ASSERT (PointersFreedOnExit != NULL) ;
+	  xrealloc (PointersFreedOnExit, sizeof(char *) * totalPointers) ;
 
       for (i = nextPointer; i < totalPointers; i++)
 	PointersFreedOnExit [i] = NULL;
@@ -700,19 +628,19 @@ char *buildFilename (const char *directory, const char *fname)
   if (directory == NULL)
     directory = "." ;
   
-  len = strlen (directory) + strlen (fname) + 2 ;
+  len = strlen (directory) + strlen (fname) + 2 + 1 ;
 
   if (len < pathMax(directory) - 2)
     {
-      p = malloc (len + 1) ;
+      p = xmalloc (len) ;
       p [0] = '\0' ;
       if (fname [0] != '/')
         {
-          strcat (p,directory) ;
+          strlcat (p,directory,len) ;
           if (p [strlen(p) - 1] != '/')
-            strcat (p,"/") ;
+            strlcat (p,"/",len) ;
         }
-      strcat (p,fname) ;
+      strlcat (p,fname,len) ;
     }
 
   return p ;
@@ -723,39 +651,44 @@ char *buildFilename (const char *directory, const char *fname)
 /* borrows heavily from the shrinkfile program by chongo. */
 bool shrinkfile (FILE *fp, long size, char *name, const char *mode)
 {
-  long currlen = ftell (fp) ;
+  long currlen = ftello (fp) ;
   char *tmpname ;
   char buffer [BUFSIZ] ;
   FILE *tmpFp ;
   int c ;
   int i ;
-
-  tmpname = malloc (pathMax(NULL) + 1) ;
-  sprintf (tmpname,"%s.XXXXXX",name) ;
-  mktemp (tmpname) ;
+  int fd ;
 
   if (currlen <= size)
     {
-      FREE(tmpname) ;
-      
       d_printf (1,"No need to shrink file (%s %ld vs %ld\n",
                name,size,currlen) ;
       return true ;
     }
 
-  /* create a temp file that will go away when closed. */
-  if ((tmpFp = fopen (tmpname,"w")) == NULL)
+  /* create a temp file. */
+  tmpname = concat (name,".XXXXXX",(char *)0) ;
+  fd = mkstemp (tmpname) ;
+
+  if (fd < 0)
     {
-      syslog (LOG_ERR,SHRINK_TEMP_OPEN,tmpname) ;
-      FREE (tmpname) ;
+      syswarn ("ME error creating temp shrink file for %s", name) ;
+      free (tmpname) ;
       return false ;
     }
 
-  if (fseek (fp,currlen - size,SEEK_SET) != 0)
+  if ((tmpFp = fdopen (fd,"w")) == NULL)
+    {
+      syswarn ("ME error opening temp shrink file %s", tmpname) ;
+      free (tmpname) ;
+      return false ;
+    }
+
+  if (fseeko (fp,currlen - size,SEEK_SET) != 0)
     {
       fclose (tmpFp) ;
-      syslog (LOG_ERR,SHRINK_SEEK,currlen - size,name) ;
-      FREE(tmpname) ;
+      warn ("ME error seeking to point %ld in %s", currlen - size, name) ;
+      free (tmpname) ;
       return false ;
     }
 
@@ -763,10 +696,10 @@ bool shrinkfile (FILE *fp, long size, char *name, const char *mode)
   while ((c = fgetc (fp)) != '\n')
     if (c == EOF)
       {
-        syslog (LOG_WARNING,SHRINK_NONL,name) ;
+        warn ("ME no newline in shrinking file %s", name) ;
         fclose (tmpFp) ;
-        fseek (fp,currlen,SEEK_SET) ;
-        FREE(tmpname) ;
+        fseeko (fp,currlen,SEEK_SET) ;
+        free (tmpname) ;
         return false ;
     }
 
@@ -776,46 +709,46 @@ bool shrinkfile (FILE *fp, long size, char *name, const char *mode)
       if (fwrite (buffer,1,i,tmpFp) != (size_t) i)
         {
           fclose (tmpFp) ;
-          syslog (LOG_ERR,SHRINK_WRITETMP,tmpname) ;
-          fseek (fp,currlen, SEEK_SET) ;
-          FREE(tmpname) ;
+          syswarn ("ME fwrite failed to temp shrink file %s", tmpname) ;
+          fseeko (fp,currlen, SEEK_SET) ;
+          free (tmpname) ;
           return false ;
         }
     }
 
   if (i < 0)
-    logAndExit (1,SHRINK_READ,name) ;
+    logAndExit (1,"ME fread failed on file %s: %s",name, strerror (errno)) ;
 
   fclose (tmpFp) ;
 
   if (unlink (name) != 0)
-    logAndExit (1,UNLINK_FAILED,name) ;
+    logAndExit (1,"ME oserr unlink %s: %s",name, strerror (errno)) ;
 
   /* we're in the same directory so this is ok. */
   if (rename (tmpname,name) != 0)
-    logAndExit (1,RENAME_FAILED,tmpname,name) ;
+    logAndExit (1,"ME oserr rename %s, %s: %s", tmpname, name,
+                strerror (errno)) ;
   
   if (freopen (name,mode,fp) != fp)
-    logAndExit (1,SHRINK_FREOPEN,name) ;
+    logAndExit (1,"ME freopen on shrink file failed %s: %s", name,
+                strerror (errno)) ;
 
-  fseek (fp,0,SEEK_END) ;
-  size = ftell (fp) ;
-  
-  syslog (LOG_WARNING,FILE_SHRUNK,name,currlen,size) ;
+  fseeko (fp,0,SEEK_END) ;
+  size = ftello (fp) ;
 
-  FREE(tmpname) ;
+  notice ("ME file %s shrunk from %ld to %ld", name, currlen, size) ;
+
+  free (tmpname) ;
   
   return true ;
 }
 
 
 
-long pathMax (const char *pathname)
+long pathMax (const char *pathname UNUSED)
 {
   static long rval = 0 ;
 
-  (void) pathname ;
-  
   if (rval > 0)
     return rval ;
   
