@@ -1,60 +1,65 @@
-/*  $Revision: 1.8 $
+/*  $Id: getmodaddr.c,v 1.22 2003/01/19 19:58:22 rra Exp $
 **
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include "configdata.h"
+
+#include "config.h"
 #include "clibrary.h"
 #include <errno.h>
-#include "paths.h"
+
+#include "inn/innconf.h"
 #include "libinn.h"
-#include "macros.h"
 #include "nntp.h"
+#include "paths.h"
 
 
-STATIC char	GMApathname[256];
-STATIC FILE	*GMAfp = NULL;
+static char	*GMApathname = NULL;
+static FILE	*GMAfp = NULL;
 
 
 /*
 **  Close the file opened by GMAlistopen.
 */
-void GMAclose(void)
+static void
+GMAclose(void)
 {
     if (GMAfp) {
-	(void)fclose(GMAfp);
+	fclose(GMAfp);
 	GMAfp = NULL;
     }
-    if (GMApathname[0]) {
-	(void)unlink(GMApathname);
-	GMApathname[0] = '\0';
+    if (GMApathname != NULL) {
+	unlink(GMApathname);
+        free(GMApathname);
+	GMApathname = NULL;
     }
 }
+
 /*
 **  Internal library routine.
 */
-FILE *GMA_listopen(char *pathname, FILE *FromServer, FILE *ToServer, char *request)
+static FILE *
+GMA_listopen(int fd, FILE *FromServer, FILE *ToServer, const char *request)
 {
     char	buff[BUFSIZ];
     char	*p;
     int		oerrno;
     FILE	*F;
 
-    (void)unlink(pathname);
-    if ((F = fopen(pathname, "w")) == NULL)
-	return NULL;
+    F = fdopen(fd, "r+");
+    if (F == NULL)
+        return NULL;
 
     /* Send a LIST command to and capture the output. */
     if (request == NULL)
-	(void)fprintf(ToServer, "list moderators\r\n");
+	fprintf(ToServer, "list moderators\r\n");
     else
-	(void)fprintf(ToServer, "list %s\r\n", request);
-    (void)fflush(ToServer);
+	fprintf(ToServer, "list %s\r\n", request);
+    fflush(ToServer);
 
     /* Get the server's reply to our command. */
     if (fgets(buff, sizeof buff, FromServer) == NULL
-     || !EQn(buff, NNTP_LIST_FOLLOWS, STRLEN(NNTP_LIST_FOLLOWS))) {
+     || strncmp(buff, NNTP_LIST_FOLLOWS, strlen(NNTP_LIST_FOLLOWS)) != 0) {
 	oerrno = errno;
+        fclose(F);
 	GMAclose();
 	errno = oerrno;
 	return NULL;
@@ -67,16 +72,16 @@ FILE *GMA_listopen(char *pathname, FILE *FromServer, FILE *ToServer, char *reque
 	if ((p = strchr(buff, '\n')) != NULL)
 	    *p = '\0';
 	if (buff[0] == '.' && buff[1] == '\0') {
-	    if (ferror(F) || fflush(F) == EOF || fclose(F) == EOF)
+	    if (ferror(F) || fflush(F) == EOF || fseeko(F, 0, SEEK_SET) != 0)
 		break;
-	    return fopen(pathname, "r");
+	    return F;
 	}
-	(void)fprintf(F, "%s\n", buff);
+	fprintf(F, "%s\n", buff);
     }
 
     /* Ran out of input before finding the terminator; quit. */
     oerrno = errno;
-    (void)fclose(F);
+    fclose(F);
     GMAclose();
     errno = oerrno;
     return NULL;
@@ -85,15 +90,19 @@ FILE *GMA_listopen(char *pathname, FILE *FromServer, FILE *ToServer, char *reque
 /*
 **  Read the moderators file, looking for a moderator.
 */
-char *GetModeratorAddress(FILE *FromServer, FILE *ToServer, char *group, char *moderatormailer)
+char *
+GetModeratorAddress(FILE *FromServer, FILE *ToServer, char *group,
+		    char *moderatormailer)
 {
     static char		address[SMBUF];
     char	        *p;
     char		*save;
+    char                *path;
     char		buff[BUFSIZ];
     char		name[SMBUF];
+    int                 fd;
 
-    (void)strcpy(name, group);
+    strlcpy(name, group, sizeof(name));
     address[0] = '\0';
 
     if (FromServer==NULL || ToServer==NULL){
@@ -102,18 +111,26 @@ char *GetModeratorAddress(FILE *FromServer, FILE *ToServer, char *group, char *m
          *  This should be part of nnrpd or the like running on the server.
          *  Open the server copy of the moderators file.
          */
-	GMAfp = fopen(cpcatpath(innconf->pathetc, _PATH_MODERATORS), "r");
+        path = concatpath(innconf->pathetc, _PATH_MODERATORS);
+	GMAfp = fopen(path, "r");
+        free(path);
     }else{
         /*
          *  Get a local copy of the moderators file from the server.
          */
-	(void)sprintf(GMApathname, "%.220s/%s", innconf->pathtmp,
-		_PATH_TEMPMODERATORS);
-        (void)mktemp(GMApathname);
-        GMAfp = GMA_listopen(GMApathname, FromServer, ToServer, "moderators");
+        GMApathname = concatpath(innconf->pathtmp, _PATH_TEMPMODERATORS);
+        fd = mkstemp(GMApathname);
+        if (fd >= 0)
+            GMAfp = GMA_listopen(fd, FromServer, ToServer, "moderators");
+        else
+            GMAfp = NULL;
+
 	/* Fallback to the local copy if the server doesn't have it */
-	if (GMAfp == NULL)
-	    GMAfp = fopen(cpcatpath(innconf->pathetc, _PATH_MODERATORS), "r");
+	if (GMAfp == NULL) {
+            path = concatpath(innconf->pathetc, _PATH_MODERATORS);
+	    GMAfp = fopen(path, "r");
+            free(path);
+        }
     }
 
     if (GMAfp != NULL) {
@@ -121,7 +138,7 @@ char *GetModeratorAddress(FILE *FromServer, FILE *ToServer, char *group, char *m
 	    /* Skip blank and comment lines. */
 	    if ((p = strchr(buff, '\n')) != NULL)
 		*p = '\0';
-	    if (buff[0] == '\0' || buff[0] == COMMENT_CHAR)
+	    if (buff[0] == '\0' || buff[0] == '#')
 		continue;
 
 	    /* Snip off the first word. */
@@ -132,18 +149,18 @@ char *GetModeratorAddress(FILE *FromServer, FILE *ToServer, char *group, char *m
 
 	    /* If it pattern-matches the newsgroup, the second field is a
 	     * format for mailing, with periods changed to dashes. */
-	    if (wildmat(name, buff)) {
+	    if (uwildmat(name, buff)) {
 		for (save = p; ISWHITE(*save); save++)
 		    continue;
 		for (p = name; *p; p++)
 		    if (*p == '.')
 			*p = '-';
-		(void)sprintf(address, save, name);
+		snprintf(address, sizeof(address), save, name);
 		break;
 	    }
 	}
 
-	(void) GMAclose();
+	 GMAclose();
 	if (address[0])
 	    return address;
     }
@@ -155,6 +172,6 @@ char *GetModeratorAddress(FILE *FromServer, FILE *ToServer, char *group, char *m
     for (p = name; *p; p++)
 	if (*p == '.')
 	    *p = '-';
-    (void)sprintf(address, save, name);
+    snprintf(address, sizeof(address), save, name);
     return address;
 }
