@@ -32,11 +32,43 @@ my %ctlinnd = ('a', 'addhist',     'D', 'allow',
 	       'E', 'logmode',     'F', 'feedinfo',
 	       'T', 'filter',      'P', 'perl',);
 
+my %timer_names = (idle     => 'idle',
+                   hishave  => 'history lookup',
+                   hisgrep  => 'history grep',
+                   hiswrite => 'history write',
+                   hissync  => 'history sync',
+                   nntpread => 'nntp read',
+                   artparse => 'article parse',
+                   artclean => 'article cleanup',
+                   artwrite => 'article write',
+                   artcncl  => 'article cancel',
+                   artlog   => 'article logging',
+                   sitesend => 'site send',
+                   overv    => 'overview write',
+                   perl     => 'perl filter',
+                   python   => 'python filter',
+                   datamove => 'data move'
+);
+
+my %innfeed_timer_names = (
+                   'idle'    => 'idle',
+		   'blstats' => 'backlog stats',
+		   'stsfile' => 'status file',
+		   'newart'  => 'article new',
+		   'prepart' => 'article prepare',
+		   'readart' => 'article read',
+		   'read'    => 'data read',
+		   'write'   => 'data write',
+		   'cb'      => 'callbacks',
+);
+
+my %nnrpd_timer_names = (
+                   'idle'    => 'idle',
+                   'newnews' => 'newnews',
+);
+
 # init innd timer
-foreach ('article cancel', 'article control', 'article link',
-	 'article write', 'history grep', 'history lookup',
-	 'history sync', 'history write', 'idle', 'site send',
-	 'overview write', 'perl filter') {
+foreach (values %timer_names) {
   $innd_time_min{$_} = $MIN;
   $innd_time_max{$_} = $MAX;
   $innd_time_time{$_} = 0;   # to avoid a warning... Perl < 5.004
@@ -44,6 +76,23 @@ foreach ('article cancel', 'article control', 'article link',
 }
 $innd_time_times = 0;        # ...
 
+# init innfeed timer
+foreach (values %innfeed_timer_names) {
+  $innfeed_time_min{$_} = $MIN;
+  $innfeed_time_max{$_} = $MAX;
+  $innfeed_time_time{$_} = 0;   # to avoid a warning... Perl < 5.004
+  $innfeed_time_num{$_} = 0;    # ...
+}
+$innfeed_time_times = 0;        # ...
+
+# init nnrpd timer
+foreach (values %nnrpd_timer_names) {
+  $nnrpd_time_min{$_} = $MIN;
+  $nnrpd_time_max{$_} = $MAX;
+  $nnrpd_time_time{$_} = 0;   # to avoid a warning... Perl < 5.004
+  $nnrpd_time_num{$_} = 0;    # ...
+}
+$nnrpd_time_times = 0;        # ...
 
 # collect: Used to collect the data.
 sub collect {
@@ -285,6 +334,8 @@ sub collect {
       $innd_accepted{$server} += $accepted;
       $innd_refused{$server} += $refused;
       $innd_rejected{$server} += $rejected;
+      $innd_stored_size{$server} += $accptsize;
+      $innd_duplicated_size{$server} += $dupsize;
       return 1;
     } elsif ($left =~ /(\S+):\d+ closed seconds (\d+) accepted (\d+) refused (\d+) rejected (\d+)$/o) {
       # closed (with times)
@@ -323,6 +374,11 @@ sub collect {
        $innd_control{"flush_file"}++;
        return 1;
      }
+    # too many connections from site
+    if ($left =~ /too many connections from (\S+)/o) {
+      $innd_max_conn{$1}++;
+      return 1;
+    }
     # overview exit 0 elapsed 23 pid 28461
     return 1 if $left =~ m/\S+ exit \d+ .*$/o;
     # internal rejecting huge article
@@ -347,115 +403,26 @@ sub collect {
       $innd_control{"throttle"}++;
       return 1;
     }
-    # time (from the Greco's patch)
-    # ME time X idle X(X) artwrite X(X) artlink X(X) hiswrite X(X) hissync
-    # X(X) sitesend X(X) artctrl X(X) artcncl X(X) hishave X(X) hisgrep X(X)
-    # perl X(X) python X(X)
-    # Note : some parameters are optional because several versions of this
-    # patch exist.
+    # profile timer
+    # ME time X nnnn X(X) [...]
+    # The exact timers change from various versions of INN, so try to deal
+    # with this in a general fashion.
     if ($left =~ m/^\S+\s+                         # ME
 	           time\ (\d+)\s+                  # time
-	           idle\ (\d+)\((\d+)\)\s+         # idle
-                   artwrite\ (\d+)\((\d+)\)\s+     # artwrite
-                   artlink\ (\d+)\((\d+)\)\s+      # artlink
-                   hiswrite\ (\d+)\((\d+)\)\s+     # hiswrite
-                   hissync\ (\d+)\((\d+)\)         # hissync
-                   (?:\s+sitesend\ (\d+)\((\d+)\)\s+  # sitesend (optional)
-                   artctrl\ (\d+)\((\d+)\)\s+      # artctrl
-                   artcncl\ (\d+)\((\d+)\)\s+      # artcncl
-                   hishave\ (\d+)\((\d+)\)         # hishave
-                   (?:\s+hisgrep\ (\d+)\((\d+)\)   # hisgrep (optional)
-		   (?:\s+perl\ (\d+)\((\d+)\)      # perl (optional)
-		   (?:\s+overv\ (\d+)\((\d+)\)     # overv (optional)
-		   (?:\s+python\ (\d+)\((\d+)\))?)?)?)?)?   # python (optional)
-	           \s*$/ox) {
+                   ((?:\S+\ \d+\(\d+\)\s*)+)       # timer values
+                   $/ox) {
       $innd_time_times += $1;
-      $innd_time_time{'idle'} += $2;
-      $innd_time_num{'idle'} += $3;
-      $innd_time_min{'idle'} = $2 / ($3 || 1)
-	if $3 && $innd_time_min{'idle'} > $2 / ($3 || 1);
-      $innd_time_max{'idle'} = $2 / ($3 || 1)
-	if $3 && $innd_time_max{'idle'} < $2 / ($3 || 1);
+      my $timers = $2;
 
-      $innd_time_time{'article write'} += $4;
-      $innd_time_num{'article write'} += $5;
-      $innd_time_min{'article write'} = $4 / ($5 || 1)
-	if $5 && $innd_time_min{'article write'} > $4 / ($5 || 1);
-      $innd_time_max{'article write'} = $4 / ($5 || 1)
-	if $5 && $innd_time_max{'article write'} < $4 / ($5 || 1);
-
-      $innd_time_time{'article link'} += $6;
-      $innd_time_num{'article link'} += $7;
-      $innd_time_min{'article link'} = $6 / ($7 || 1)
-	if $7 && $innd_time_min{'article link'} > $6 / ($7 || 1);
-      $innd_time_max{'article link'} = $6 / ($7 || 1)
-	if $7 && $innd_time_max{'article link'} < $6 / ($7 || 1);
-
-      $innd_time_time{'history write'} += $8;
-      $innd_time_num{'history write'} += $9;
-      $innd_time_min{'history write'} = $8 / ($9 || 1)
-	if $9 && $innd_time_min{'history write'} > $8 / ($9 || 1);
-      $innd_time_max{'history write'} = $8 / ($9 || 1)
-	if $9 && $innd_time_max{'history write'} < $8 / ($9 || 1);
-
-      $innd_time_time{'history sync'} += $10;
-      $innd_time_num{'history sync'} += $11;
-      $innd_time_min{'history sync'} = $10 / ($11 || 1)
-	if $11 && $innd_time_min{'history sync'} > $10 / ($11 || 1);
-      $innd_time_max{'history sync'} = $10 / ($11 || 1)
-	if $11 && $innd_time_max{'history sync'} < $10 / ($11 || 1);
-
-      $innd_time_time{'site send'} += $12;
-      $innd_time_num{'site send'} += $13;
-      $innd_time_min{'site send'} = $12 / ($13 || 1)
-	if $13 && $innd_time_min{'site send'} > $12 / ($13 || 1);
-      $innd_time_max{'site send'} = $12 / ($13 || 1)
-	if $13 && $innd_time_max{'site send'} < $12 / ($13 || 1);
-
-      $innd_time_time{'article control'} += $14;
-      $innd_time_num{'article control'} += $15;
-      $innd_time_min{'article control'} = $14 / ($15 || 1)
-	if $15 && $innd_time_min{'article control'} > $14 / ($15 || 1);
-      $innd_time_max{'article control'} = $14 / ($15 || 1)
-	if $15 && $innd_time_max{'article control'} < $14 / ($15 || 1);
-
-      $innd_time_time{'article cancel'} += $16;
-      $innd_time_num{'article cancel'} += $17;
-      $innd_time_min{'article cancel'} = $16 / ($17 || 1)
-	if $17 && $innd_time_min{'article cancel'} > $16 / ($17 || 1);
-      $innd_time_max{'article cancel'} = $16 / ($17 || 1)
-	if $17 && $innd_time_max{'article cancel'} < $16 / ($17 || 1);
-
-      $innd_time_time{'history lookup'} += $18;
-      $innd_time_num{'history lookup'} += $19;
-      $innd_time_min{'history lookup'} = $18 / ($19 || 1)
-	if $19 && $innd_time_min{'history lookup'} > $18 / ($19 || 1);
-      $innd_time_max{'history lookup'} = $18 / ($19 || 1)
-	if $19 && $innd_time_max{'history lookup'} < $18 / ($19 || 1);
-
-      if (defined($20) && defined($21)) {
-        $innd_time_time{'history grep'} += $20;
-        $innd_time_num{'history grep'} += $21;
-        $innd_time_min{'history grep'} = $20 / ($21 || 1)
-	  if $21 && $innd_time_min{'history grep'} > $20 / ($21 || 1);
-        $innd_time_max{'history grep'} = $20 / ($21 || 1)
-	  if $21 && $innd_time_max{'history grep'} < $20 / ($21 || 1);
-        if (defined($22) && defined($23)) {
-          $innd_time_time{'perl filter'} += $22;
-          $innd_time_num{'perl filter'} += $23;
-          $innd_time_min{'perl filter'} = $22 / ($23 || 1)
-	    if $23 && $innd_time_min{'perl filter'} > $22 / ($23 || 1);
-          $innd_time_max{'perl filter'} = $22 / ($23 || 1)
-	    if $23 && $innd_time_max{'perl filter'} < $22 / ($23 || 1);
-	  if (defined($24) && defined($25))  {
-	    $innd_time_time{'overview write'} += $24;
-	    $innd_time_num{'overview write'} += $25;
-	    $innd_time_min{'overview write'} = $24 / ($25 || 1)
-	      if $25 && $innd_time_min{'overview write'} > $24 / ($25 || 1);
-	    $innd_time_max{'overview write'} = $24 / ($25 || 1)
-	      if $25 && $innd_time_max{'overview write'} < $24 / ($25 || 1);
-	  }
-        }
+      while ($timers =~ /(\S+) (\d+)\((\d+)\)\s*/g) {
+        my $name = $timer_names{$1} || $1;
+        my $average = $2 / ($3 || 1);
+        $innd_time_time{$name} += $2;
+        $innd_time_num{$name} += $3;
+        $innd_time_min{$name} = $average
+          if ($3 && $innd_time_min{$name} > $average);
+        $innd_time_max{$name} = $average
+          if ($3 && $innd_time_max{$name} < $average);
       }
       return 1;
     }
@@ -647,6 +614,8 @@ sub collect {
     return 1 if $left =~ /CNFS-sm: cycbuff \S+ rollover to cycle/o;
     # CNFS-sm: CNFSflushallheads: flushing - possibly interesting
     return 1 if $left =~ /CNFS-sm: CNFSflushallheads: flushing /o;
+    # CNFS-sm: metacycbuff rollover with SEQUENTIAL
+    return 1 if $left =~ /CNFS-sm: metacycbuff \S+ cycbuff is moved to /o;
     # Cleanfeed status reports
     return 1 if $left =~ /^filter: status/o;
   }
@@ -713,6 +682,28 @@ sub collect {
       $innfeed_shrunk{$file} += $s1 - $s2;
       return 1;
     }
+    # profile timer
+    # ME time X nnnn X(X) [...]
+    return 1 if $left =~ m/backlogstats/;
+    if ($left =~ m/^\S+\s+                         # ME
+                   time\ (\d+)\s+                  # time
+                   ((?:\S+\ \d+\(\d+\)\s*)+)       # timer values
+                   $/ox) {
+      $innfeed_time_times += $1;
+      my $timers = $2;
+
+      while ($timers =~ /(\S+) (\d+)\((\d+)\)\s*/g) {
+        my $name = $innfeed_timer_names{$1} || $1;
+        my $average = $2 / ($3 || 1);
+        $innfeed_time_time{$name} += $2;
+        $innfeed_time_num{$name} += $3;
+        $innfeed_time_min{$name} = $average
+          if ($3 && $innfeed_time_min{$name} > $average);
+        $innfeed_time_max{$name} = $average
+          if ($3 && $innfeed_time_max{$name} < $average);
+      }
+      return 1;
+    }
     # xxx grabbing external tape file
     return 1 if $left =~ m/ grabbing external tape file/o;
     # hostChkCxns - maxConnections was
@@ -731,6 +722,10 @@ sub collect {
     return 1 if $left =~ m/(?:SERVER|ME) articles active \d+ bytes \d+/o;
     # connect : Connection refused
     return 1 if $left =~ m/connect : Connection refused/o;
+    # connect : Network is unreachable
+    return 1 if $left =~ m/connect : Network is unreachable/o;
+    # connect : Address family not supported by protocol
+    return 1 if $left =~ m/connect : Address family not supported by protocol/o;
     # connect : No route to host
     return 1 if $left =~ m/connect : No route to host/o;
     # connection vanishing
@@ -1201,7 +1196,7 @@ sub collect {
   }
   ########
   ## nnrpd
-  if ($prog eq "nnrpd")
+  if ($prog =~ /^nnrpd(?:-ssl)?$/)
   {
     # Fix a small bug of nnrpd (inn 1.4*)
     $left =~ s/^ /\? /o;
@@ -1218,7 +1213,7 @@ sub collect {
     # perl filtering enabled
     return 1 if $left =~ /perl filtering enabled$/o;
     # connect
-    if ($left =~ /(\S+) connect$/o) {
+    if ($left =~ /(\S+) (\([0-9a-fA-F:.]*\) )?connect$/o) {
       my $cust = $1;
       $cust = lc $cust unless $CASE_SENSITIVE;
       my $dom = &host2dom($cust);
@@ -1282,6 +1277,12 @@ sub collect {
       $nnrpd_no_permission{$cust}++;
       return 1;
     }
+    # authinfo
+    if ($left =~ /\S+ user (\S+)$/o) {
+      my $user = $1;
+      $nnrpd_auth{$user}++;
+      return 1;
+    }
     # unrecognized + command
     if ($left =~ /(\S+) unrecognized (.*)$/o) {
       my ($cust, $error) = ($1, $2);
@@ -1307,11 +1308,15 @@ sub collect {
       return 1;
     }
     # times
-    if ($left =~ /(\S+) times user (\S+) system (\S+) elapsed (\S+)$/o) {
-      my ($cust, $user, $system, $elapsed) = ($1, $2, $3, $4);
+    if ($left =~ /(\S+) times user (\S+) system (\S+) idle (\S+) elapsed (\S+)$/o) {
+      my ($cust, $user, $system, $idle, $elapsed) = ($1, $2, $3, $4, $5);
       $cust = lc $cust unless $CASE_SENSITIVE;
       my $dom = &host2dom($cust);
       $nnrpd_times{$cust} += $elapsed;
+      $nnrpd_resource_user{$cust} += $user;
+      $nnrpd_resource_system{$cust} += $system;
+      $nnrpd_resource_idle{$cust} += $idle;
+      $nnrpd_resource_elapsed{$cust} += $elapsed;
       $nnrpd_dom_times{$dom} += $elapsed;
       return 1;
     }
@@ -1387,6 +1392,29 @@ sub collect {
       $nnrpd_gethostbyaddr{"? (can't getpeername)"}++;
       return 1;
     }
+    # profile timer
+    # ME time X nnnn X(X) [...]
+    # The exact timers change from various versions of INN, so try to deal
+    # with this in a general fashion.
+    if ($left =~ m/^\S+\s+                         # ME
+	           time\ (\d+)\s+                  # time
+                   ((?:\S+\ \d+\(\d+\)\s*)+)       # timer values
+                   $/ox) {
+      $nnrpd_time_times += $1;
+      my $timers = $2;
+
+      while ($timers =~ /(\S+) (\d+)\((\d+)\)\s*/g) {
+        my $name = $nnrpd_timer_names{$1} || $1;
+        my $average = $2 / ($3 || 1);
+        $nnrpd_time_time{$name} += $2;
+        $nnrpd_time_num{$name} += $3;
+        $nnrpd_time_min{$name} = $average
+          if ($3 && $nnrpd_time_min{$name} > $average);
+        $nnrpd_time_max{$name} = $average
+          if ($3 && $nnrpd_time_max{$name} < $average);
+      }
+      return 1;
+    }
     # ME dropping articles into ...
     return 1 if $left = ~/ME dropping articles into /o;
     # newnews (interesting but ignored till now)
@@ -1413,9 +1441,8 @@ sub collect {
   ## overchan
   if ($prog eq "overchan") {
     # times
-    if ($left =~ /(\S+) timings (\d+) arts (\d+) of (\d+) ms$/o) {
-      my ($server, $articles, $work_time, $run_time) = ($1, $2, $3, $4);
-      $server = lc $server unless $CASE_SENSITIVE;
+    if ($left =~ /timings (\d+) arts (\d+) of (\d+) ms$/o) {
+      my ($articles, $work_time, $run_time) = ($1, $2, $3);
       # ??? What to do with numbers
       return 1;
     }
@@ -1649,7 +1676,7 @@ sub collect {
     if ($left =~ m|^Class\ (\S+)\ for\ groups\ matching\ \S+
                     (\ article\ size\ min/max:\ \d+/\d+)?
                     \ Buffer\ (\S+),
-                    \ len:\ (\d+)\s+Mbytes,
+                    \ len:\ ([\d.]+)\s+Mbytes,
                     \ used:\ ([\d.]+)\ Mbytes\ \(\s*[\d.]+%\)
                     \s+(\d+)\ cycles\s*
                  $|ox) {
@@ -1727,11 +1754,11 @@ sub adjust {
 	  $nnrpd_connect{$serv} -= $nnrpd_no_permission{$serv};
 	  $nnrpd_groups{$serv} -= $nnrpd_no_permission{$serv}
 	    if defined $nnrpd_groups{$serv};
-	  $nnrpd_times{$serv} -= $nnrpd_no_permission{$serv}
-	    if defined $nnrpd_times{$serv};
 	  delete $nnrpd_connect{$serv} unless $nnrpd_connect{$serv};
 	  delete $nnrpd_groups{$serv}  unless $nnrpd_groups{$serv};
 	  delete $nnrpd_times{$serv}   unless $nnrpd_times{$serv};
+	  delete $nnrpd_usr_times{$serv}   unless $nnrpd_usr_times{$serv};
+	  delete $nnrpd_sys_times{$serv}   unless $nnrpd_sys_times{$serv};
 	  delete $nnrpd_dom_connect{$dom} unless $nnrpd_dom_connect{$dom};
 	  delete $nnrpd_dom_groups{$dom}  unless $nnrpd_dom_groups{$dom};
 	  delete $nnrpd_dom_times{$dom}   unless $nnrpd_dom_times{$dom};
@@ -1755,6 +1782,8 @@ sub adjust {
       $innd_offered{$key} = ($innd_accepted{$key} || 0)
 	+ ($innd_refused{$key} || 0)
 	+ ($innd_rejected{$key} || 0);
+      $innd_offered_size{$key} = ($innd_stored_size{$key} || 0)
+	+ ($innd_duplicated_size{$key} || 0);
     }
 
 
@@ -1768,7 +1797,18 @@ sub adjust {
 	#$innd_time_max{$key} /= 1000;
       }
     }
-
+    if (%innfeed_time_min) {
+      foreach $key (keys (%innfeed_time_min)) {
+        $innfeed_time_min{$key} = 0 if ($innfeed_time_min{$key} == $MIN);
+        $innfeed_time_max{$key} = 0 if ($innfeed_time_max{$key} == $MAX);
+      }
+    }
+    if (%nnrpd_time_min) {
+      foreach $key (keys (%nnrpd_time_min)) {
+        $nnrpd_time_min{$key} = 0 if ($nnrpd_time_min{$key} == $MIN);
+        $nnrpd_time_max{$key} = 0 if ($nnrpd_time_max{$key} == $MAX);
+      }
+    }
     # remove the innd timer stats if not used.
     unless ($innd_time_times) {
       undef %innd_time_min;
@@ -1776,6 +1816,20 @@ sub adjust {
       undef %innd_time_num;
       undef %innd_time_time;
     }
+    # same thing for innfeed timer
+    unless ($innfeed_time_times) {
+      undef %innfeed_time_min;
+      undef %innfeed_time_max;
+      undef %innfeed_time_num;
+      undef %innfeed_time_time;
+    }
+    # same thing for nnrpd timer
+    unless ($nnrpd_time_times) {
+      undef %nnrpd_time_min;
+      undef %nnrpd_time_max;
+      undef %nnrpd_time_num;
+      undef %nnrpd_time_time;
+    }
 
     # adjust the crosspost stats.
     if (%crosspost) {
