@@ -1,4 +1,4 @@
-/*  $Id: overdata.c 9019 2010-03-19 21:27:15Z iulius $
+/*  $Id: overdata.c,v 1.12.2.2 2004/05/16 20:26:55 rra Exp $
 **
 **  Overview data processing.
 **
@@ -13,20 +13,22 @@
 #include "inn/buffer.h"
 #include "inn/innconf.h"
 #include "inn/messages.h"
+#include "inn/qio.h"
 #include "inn/wire.h"
 #include "inn/vector.h"
-#include "inn/libinn.h"
+#include "libinn.h"
 #include "ovinterface.h"
+#include "paths.h"
 
 
-/* The standard overview fields.  The order of these fields is important. */
+/* The standard overview fields. */
 static const char * const fields[] = {
     "Subject", "From", "Date", "Message-ID", "References", "Bytes", "Lines"
 };
 
 
 /*
-**  Return a vector of the standard overview fields.  Note there is no
+**  Return a vector of the standard overview fields. Note there is no
 **  way to free up the resulting data structure.
 */
 const struct cvector *
@@ -48,50 +50,68 @@ overview_fields(void)
 }
 
 /*
-**  Return a vector of the additional fields over the standard ones.
-**  The order of these fields is important.
-**
-**  Xref: is mandatory for INN and we make it the first extra field
-**  after the seven overview fields defined in RFC 3977.
-**
-**  Caller is responsible for freeing the vector.
+**  Parse the overview schema and return a vector of the additional fields
+**  over the standard ones.  Caller is responsible for freeing the vector.
 */
 struct vector *
-overview_extra_fields(bool hidden)
+overview_extra_fields(void)
 {
-    struct vector *list = NULL;
-    unsigned int i;
+    struct vector *list;
+    char *schema = NULL;
+    char *line, *p;
+    QIOSTATE *qp = NULL;
+    unsigned int field;
+    bool full = false;
 
+    schema = concatpath(innconf->pathetc, _PATH_SCHEMA);
+    qp = QIOopen(schema);
+    if (qp == NULL) {
+        syswarn("cannot open %s", schema);
+        goto fail;
+    }
     list = vector_new();
-
-    if (hidden) {
-        vector_resize(list, innconf->extraoverviewadvertised->count
-                            + innconf->extraoverviewhidden->count + 1);
-    } else {
-        vector_resize(list, innconf->extraoverviewadvertised->count + 1);
+    for (field = 0, line = QIOread(qp); line != NULL; line = QIOread(qp)) {
+        while (ISWHITE(*line))
+            line++;
+        p = strchr(line, '#');
+        if (p != NULL)
+            *p = '\0';
+        p = strchr(line, '\n');
+        if (p != NULL)
+            *p = '\0';
+        if (*line == '\0')
+            continue;
+        p = strchr(line, ':');
+        if (p != NULL) {
+            *p++ = '\0';
+            full = (strcmp(p, "full") == 0);
+        }
+        if (field >= ARRAY_SIZE(fields)) {
+            if (!full)
+                warn("additional field %s not marked with :full", line);
+            vector_add(list, line);
+        } else {
+            if (strcasecmp(line, fields[field]) != 0)
+                warn("field %d is %s, should be %s", field, line,
+                     fields[field]);
+        }
+        field++;
     }
-
-    vector_add(list, "Xref");
-
-    if (innconf->extraoverviewadvertised->strings != NULL) {
-        for (i = 0; i < innconf->extraoverviewadvertised->count; i++) {
-            if (innconf->extraoverviewadvertised->strings[i] != NULL) {
-                vector_add(list, innconf->extraoverviewadvertised->strings[i]);
-            }
+    if (QIOerror(qp)) {
+        if (QIOtoolong(qp)) {
+            warn("line too long in %s", schema);
+        } else {
+            syswarn("error while reading %s", schema);
         }
     }
-
-    if (hidden) {
-        if (innconf->extraoverviewhidden->strings != NULL) {
-            for (i = 0; i < innconf->extraoverviewhidden->count; i++) {
-                if (innconf->extraoverviewhidden->strings[i] != NULL) {
-                    vector_add(list, innconf->extraoverviewhidden->strings[i]);
-                }
-            }
-        }
-    }
-
     return list;
+
+ fail:
+    if (schema != NULL)
+        free(schema);
+    if (qp != NULL)
+        QIOclose(qp);
+    return NULL;
 }
 
 
@@ -109,31 +129,13 @@ build_header(const char *article, size_t length, const char *header,
     size_t offset;
     const char *data, *end, *p;
 
-    data = wire_findheader(article, length, header, false);
+    data = wire_findheader(article, length, header);
     if (data == NULL)
         return;
     end = wire_endheader(data, article + length - 1);
     if (end == NULL)
         return;
 
-    /* Someone managed to break their server so that they were appending
-       multiple Xref headers, and INN had a bug where it wouldn't notice this
-       and reject the article.  Just in case, see if there are multiple Xref
-       headers and use the last one. */
-    if (strcasecmp(header, "xref") == 0) {
-        const char *next = end + 1;
-
-        while (next != NULL) {
-            next = wire_findheader(next, length - (next - article), header, false);
-            if (next != NULL) {
-                data = next;
-                end = wire_endheader(data, article + length - 1);
-                if (end == NULL)
-                    return;
-            }
-        }
-    }
-
     size = end - data + 1;
     offset = overview->used + overview->left;
     buffer_resize(overview, offset + size);
@@ -177,14 +179,12 @@ overview_build(ARTNUM number, const char *article, size_t length,
         } else
             build_header(article, length, fields[field], overview);
     }
-    if (extra != NULL) {
-        for (field = 0; field < extra->count; field++) {
-            buffer_append(overview, "\t", 1);
-            buffer_append(overview, extra->strings[field],
-                          strlen(extra->strings[field]));
-            buffer_append(overview, ": ", 2);
-            build_header(article, length, extra->strings[field], overview);
-        }
+    for (field = 0; field < extra->count; field++) {
+        buffer_append(overview, "\t", 1);
+        buffer_append(overview, extra->strings[field],
+                      strlen(extra->strings[field]));
+        buffer_append(overview, ": ", 2);
+        build_header(article, length, extra->strings[field], overview);
     }
     buffer_append(overview, "\r\n", 2);
     return overview;
@@ -200,7 +200,7 @@ valid_number(const char *string)
     const char *p;
 
     for (p = string; *p != '\0'; p++)
-        if (!isdigit((unsigned char) *p))
+        if (!CTYPE(isdigit, *p))
             return false;
     return true;
 }
@@ -208,7 +208,7 @@ valid_number(const char *string)
 
 /*
 **  Check whether a given string is a valid overview string (doesn't contain
-**  CR or LF, and if the second argument is true must be preceded by a header
+**  CR or LF, and if the second argument is true must be preceeded by a header
 **  name, colon, and space).  Allow CRLF at the end of the data, but don't
 **  require it.
 */
@@ -217,15 +217,13 @@ valid_overview_string(const char *string, bool full)
 {
     const unsigned char *p;
 
-    /* RFC 5322 says that header fields must consist of printable ASCII
+    /* RFC 2822 says that header fields must consist of printable ASCII
        characters (characters between 33 and 126, inclusive) excluding colon.
        We also allow high-bit characters, just in case, but not DEL. */
     p = (const unsigned char *) string;
     if (full) {
-        if (*p == '\0' || (*p == '\r' && p[1] == '\n' && p[2] == '\0'))
-            return true;
         for (; *p != '\0' && *p != ':'; p++)
-            if (!isgraph((unsigned char) *p))
+            if (*p < 33 || *p == 127)
                 return false;
         if (*p != ':')
             return false;
@@ -234,9 +232,9 @@ valid_overview_string(const char *string, bool full)
             return false;
     }
     for (p++; *p != '\0'; p++) {
-        if (*p == '\r' && p[1] == '\n' && p[2] == '\0')
+        if (*p == '\015' && p[1] == '\012' && p[2] == '\0')
             break;
-        if (*p == '\r' || *p == '\n')
+        if (*p == '\015' || *p == '\012')
             return false;
     }
     return true;
@@ -245,7 +243,7 @@ valid_overview_string(const char *string, bool full)
 
 /*
 **  Check the given overview data and make sure it's well-formed.  Extension
-**  headers are not checked against LIST OVERVIEW.FMT (having a different set of
+**  headers are not checked against overview.fmt (having a different set of
 **  extension headers doesn't make the data invalid), but the presence of the
 **  standard fields is checked.  Also checked is whether the article number in
 **  the data matches the passed article number.  Returns true if the data is
@@ -294,21 +292,23 @@ overview_check(const char *data, size_t length, ARTNUM article)
 
 
 /*
-**  Given an overview header, return the offset of the field within the
-**  overview data, or -1 if the field is not present in the overview schema
-**  for this installation.
+**  Given an overview header, return the offset of the field within
+**  the overview data, or -1 if the field is not present in the
+**  overview schema for this installation.
 */
 int
 overview_index(const char *field, const struct vector *extra)
 {
-    size_t i;
+    int i;
 
-    for (i = 0; i < ARRAY_SIZE(fields); i++)
+    for (i = 0; i < (sizeof fields / sizeof fields[0]); ++i) {
 	if (strcasecmp(field, fields[i]) == 0)
 	    return i;
-    for (i = 0; i < extra->count; i++)
+    }
+    for (i = 0; i < extra->count; i++) {
 	if (strcasecmp(field, extra->strings[i]) == 0)
-	    return i + ARRAY_SIZE(fields);
+	    return i + (sizeof fields / sizeof fields[0]);
+    }
     return -1;
 }
 
@@ -316,11 +316,11 @@ overview_index(const char *field, const struct vector *extra)
 /*
 **  Given an overview header line, split out a vector pointing at each
 **  of the components (within line), returning a pointer to the
-**  vector.  If the vector initially passed in is NULL, a new vector is
-**  created, otherwise the existing one is filled in.
+**  vector. If the vector initially passed in is NULL a new vector is
+**  created, else the existing one is filled in.
 **
 **  A member `n' of the vector is of length (vector->strings[n+1] -
-**  vector->strings[n] - 1).  Note that the last member of the vector
+**  vector->strings[n] - 1). Note that the last member of the vector
 **  will always point beyond (line + length).
 */
 struct cvector *
@@ -335,7 +335,7 @@ overview_split(const char *line, size_t length, ARTNUM *number,
 	cvector_clear(vector);
     }
     while (line != NULL) {
-	/* The first field is the article number. */
+	/* the first field is the article number */
 	if (p == NULL) {
 	    if (number != NULL) {
 		*number = atoi(line);
@@ -345,15 +345,15 @@ overview_split(const char *line, size_t length, ARTNUM *number,
 	}
 	p = memchr(line, '\t', length);
 	if (p != NULL) {
-	    /* Skip over the tab. */
+	    /* skip over the tab */
 	    ++p;
-	    /* And calculate the remaining length. */
+	    /* and calculate the remaining length */
 	    length -= (p - line);
 	} else {
-	    /* Add in a pointer to beyond the end of the final
-	       component, so you can always calculate the length;
-	       overview lines are always terminated with \r\n, so the
-	       -1 ends up chopping those off. */
+	    /* add in a pointer to beyond the end of the final
+	     * component, so you can always calculate the length.
+	     * overview lines are always terminated with \r\n, so the
+	     * -1 ends up chopping those off */
 	    cvector_add(vector, line + length - 1);
 	}
 	line = p;
@@ -364,61 +364,32 @@ overview_split(const char *line, size_t length, ARTNUM *number,
 /*
 **  Given an overview vector (from overview_split), return a copy of
 **  the member which the caller is interested in (and must free).
-**  The name and order of standard overview fields are fixed so we
-**  can request them by their index.
 */
 char *
-overview_get_standard_header(const struct cvector *vector, unsigned int element)
+overview_getheader(const struct cvector *vector, int element,
+		   const struct vector *extra)
 {
     char *field = NULL;
     size_t len;
     const char *p;
-    size_t max = ARRAY_SIZE(fields) - 1;
 
-    /* vector ends with an additional pointer beyond the end of the final
-       component.  It therefore has a count of elements increased by 1. */
-    if ((element + 1) >= vector->count || element > max) {
-        warn("request for invalid standard overview field %d", element);
-        return NULL;
+    if ((element + 1) >= vector->count ||
+	(element >= ARRAY_SIZE(fields) &&
+	 (element - ARRAY_SIZE(fields)) >= extra->count)) {
+	warn("request for invalid overview field %d", element);
+	return NULL;
+    }
+    /* Note... this routine does not synthesise Newsgroups: on behalf
+     * of the caller... */
+    if (element >= ARRAY_SIZE(fields)) {
+	/* +2 for `: ' */
+	p = vector->strings[element] +
+	    strlen(extra->strings[element - ARRAY_SIZE(fields)]) + 2;
+	len = vector->strings[element + 1] - p - 1;
+    } else {
+	p = vector->strings[element];
+	len = vector->strings[element + 1] - vector->strings[element] - 1;
     }
-
-    p = vector->strings[element];
-    len = vector->strings[element + 1] - vector->strings[element] - 1;
-
     field = xstrndup(p, len);
     return field;
 }
-
-/*
-**  Given an overview vector (from overview_split), return a copy of
-**  the member which the caller is interested in (and must free).
-**  The order of extra overview fields may vary so we walk all the
-**  extra headers to find the requested field.
-*/
-char *
-overview_get_extra_header(const struct cvector *vector, const char *header)
-{
-    char *field = NULL;
-    size_t i, len;
-    const char *p;
-    size_t headerlen = strlen(header);
-
-    /* vector ends with an additional pointer beyond the end of the final
-       component.  It therefore has a count of elements increased by 1. */
-    for (i = ARRAY_SIZE(fields); i < vector->count - 1; i++) {
-        if (strncasecmp(header, vector->strings[i], headerlen) == 0) {
-            p = vector->strings[i] + headerlen;
-            /* Check for ": " after the name of the header. */
-            if ((*p++ == ':') && (*p++ == ' ')) {
-                len = vector->strings[i + 1] - p - 1;
-
-                field = xstrndup(p, len);
-                return field;
-            }
-        }
-    }
-
-    /* The required header was not found in the extra overview fields. */
-    return NULL;
-}
-
