@@ -1,4 +1,4 @@
-/*  $Id: timehash.h,v 1.8.2.2 2001/02/03 07:28:53 rra Exp $
+/*  $Id: timehash.h,v 1.12 2001/01/04 06:02:50 rra Exp $
 **
 **  timehash based storing method header
 */
@@ -6,17 +6,17 @@
 #ifndef __TIMEHASH_H__
 #define __TIMEHASH_H__
 
-#include <configdata.h>
-#include <interface.h>
+#include "config.h"
+#include "interface.h"
 
-BOOL timehash_init(SMATTRIBUTE *attr);
+bool timehash_init(SMATTRIBUTE *attr);
 TOKEN timehash_store(const ARTHANDLE article, const STORAGECLASS class);
 ARTHANDLE *timehash_retrieve(const TOKEN token, const RETRTYPE amount);
 ARTHANDLE *timehash_next(const ARTHANDLE *article, const RETRTYPE amount);
 void timehash_freearticle(ARTHANDLE *article);
-BOOL timehash_cancel(TOKEN token);
-BOOL timehash_ctl(PROBETYPE type, TOKEN *token, void *value);
-BOOL timehash_flushcacheddata(FLUSHTYPE type);
+bool timehash_cancel(TOKEN token);
+bool timehash_ctl(PROBETYPE type, TOKEN *token, void *value);
+bool timehash_flushcacheddata(FLUSHTYPE type);
 void timehash_printfiles(FILE *file, TOKEN token, char **xref, int ngroups);
 void timehash_shutdown(void);
 
